@@ -888,9 +888,20 @@ static int state_check_process(struct snapraid_state* state, int fix, struct sna
 	char esc_buffer[ESC_MAX];
 	char esc_buffer_alt[ESC_MAX];
 	bit_vect_t* block_enabled;
+	struct snapraid_bw bw;
 
 	handle = handle_mapping(state, &diskmax);
 
+	/* initialize the bandwith context */
+	bw_init(&bw, state->opt.bwlimit);
+
+	/* share the bandwidth context with all handles */
+	for (j = 0; j < diskmax; ++j)
+		handle[j].bw = &bw;
+	for (j = 0; j < state->level; ++j)
+		if (parity[j])
+			parity[j]->bw = &bw;
+
 	/* we need 1 * data + 2 * parity + 1 * zero */
 	buffermax = diskmax + 2 * state->level + 1;
 
@@ -931,7 +942,9 @@ static int state_check_process(struct snapraid_state* state, int fix, struct sna
 	/* check all the blocks in files */
 	countsize = 0;
 	countpos = 0;
-	state_progress_begin(state, blockstart, blockmax, countmax);
+	if (!state_progress_begin(state, blockstart, blockmax, countmax))
+		goto end;
+
 	for (i = blockstart; i < blockmax; ++i) {
 		unsigned failed_count;
 		int valid_parity;
@@ -1445,6 +1458,21 @@ static int state_check_process(struct snapraid_state* state, int fix, struct sna
 			break;
 			/* LCOV_EXCL_STOP */
 		}
+
+		/* thermal control */
+		if (state_thermal_alarm(state)) {
+			/* until now is misc */
+			state_usage_misc(state);
+
+			state_progress_stop(state);
+
+			state_thermal_cooldown(state);
+
+			state_progress_restart(state);
+
+			/* drop until now */
+			state_usage_waste(state);
+		}
 	}
 
 	/* for each disk, recover empty files, symlinks and empty dirs */
@@ -1805,7 +1833,8 @@ static int state_check_process(struct snapraid_state* state, int fix, struct sna
 		}
 	}
 
-	state_progress_end(state, countpos, countmax, countsize);
+end:
+	state_progress_end(state, countpos, countmax, countsize, "Nothing to check.\n");
 
 bail:
 	/* close all the files left open */
