@@ -241,70 +241,6 @@ static double SMART_188_R[SMART_MEASURES] = {
 	1.3731,
 };
 
-/*
- * Divider for SMART attribute 193
- */
-static unsigned SMART_193_STEP = 649;
-
-/*
- * Failure rate for 30 days, for a disk
- * with SMART attribute 193 at a given value.
- */
-static double SMART_193_R[SMART_MEASURES] = {
-	0.0000, 0.0016, 0.0032, 0.0036, 0.0039,
-	0.0042, 0.0046, 0.0049, 0.0052, 0.0054,
-	0.0057, 0.0060, 0.0062, 0.0065, 0.0068,
-	0.0071, 0.0074, 0.0077, 0.0080, 0.0083,
-	0.0086, 0.0091, 0.0094, 0.0098, 0.0101,
-	0.0104, 0.0108, 0.0111, 0.0119, 0.0122,
-	0.0127, 0.0130, 0.0134, 0.0137, 0.0141,
-	0.0144, 0.0146, 0.0152, 0.0155, 0.0159,
-	0.0163, 0.0165, 0.0168, 0.0172, 0.0176,
-	0.0179, 0.0184, 0.0188, 0.0190, 0.0194,
-	0.0197, 0.0201, 0.0204, 0.0207, 0.0209,
-	0.0213, 0.0215, 0.0219, 0.0221, 0.0225,
-	0.0229, 0.0234, 0.0241, 0.0246, 0.0253,
-	0.0263, 0.0278, 0.0286, 0.0293, 0.0298,
-	0.0302, 0.0306, 0.0311, 0.0315, 0.0319,
-	0.0322, 0.0329, 0.0334, 0.0338, 0.0343,
-	0.0348, 0.0352, 0.0358, 0.0362, 0.0367,
-	0.0371, 0.0374, 0.0378, 0.0383, 0.0388,
-	0.0393, 0.0397, 0.0401, 0.0404, 0.0410,
-	0.0416, 0.0422, 0.0428, 0.0436, 0.0443,
-	0.0449, 0.0454, 0.0457, 0.0462, 0.0468,
-	0.0473, 0.0479, 0.0483, 0.0488, 0.0491,
-	0.0493, 0.0497, 0.0500, 0.0504, 0.0507,
-	0.0510, 0.0514, 0.0519, 0.0523, 0.0528,
-	0.0533, 0.0538, 0.0542, 0.0547, 0.0551,
-	0.0556, 0.0560, 0.0565, 0.0572, 0.0577,
-	0.0584, 0.0590, 0.0594, 0.0599, 0.0603,
-	0.0607, 0.0611, 0.0616, 0.0621, 0.0626,
-	0.0632, 0.0639, 0.0647, 0.0655, 0.0661,
-	0.0669, 0.0676, 0.0683, 0.0691, 0.0699,
-	0.0708, 0.0713, 0.0719, 0.0724, 0.0730,
-	0.0736, 0.0745, 0.0751, 0.0759, 0.0769,
-	0.0779, 0.0787, 0.0796, 0.0804, 0.0815,
-	0.0825, 0.0833, 0.0840, 0.0847, 0.0854,
-	0.0859, 0.0865, 0.0873, 0.0881, 0.0890,
-	0.0900, 0.0912, 0.0919, 0.0929, 0.0942,
-	0.0956, 0.0965, 0.0976, 0.0986, 0.0995,
-	0.1006, 0.1019, 0.1031, 0.1038, 0.1045,
-	0.1051, 0.1058, 0.1066, 0.1072, 0.1077,
-	0.1084, 0.1091, 0.1099, 0.1104, 0.1111,
-	0.1118, 0.1127, 0.1135, 0.1142, 0.1149,
-	0.1157, 0.1163, 0.1168, 0.1173, 0.1179,
-	0.1184, 0.1189, 0.1195, 0.1203, 0.1208,
-	0.1213, 0.1223, 0.1231, 0.1240, 0.1246,
-	0.1252, 0.1260, 0.1269, 0.1276, 0.1287,
-	0.1303, 0.1311, 0.1319, 0.1328, 0.1335,
-	0.1341, 0.1348, 0.1362, 0.1373, 0.1380,
-	0.1387, 0.1392, 0.1398, 0.1403, 0.1408,
-	0.1412, 0.1418, 0.1422, 0.1428, 0.1434,
-	0.1439, 0.1445, 0.1451, 0.1457, 0.1464,
-	0.1469, 0.1475, 0.1480, 0.1486, 0.1491,
-	0.1498,
-};
-
 /*
  * Divider for SMART attribute 197
  */
@@ -501,12 +437,6 @@ static double smart_afr(uint64_t* smart, const char* model)
 			afr = r;
 	}
 
-	if (smart[193] != SMART_UNASSIGNED) {
-		double r = smart_afr_value(SMART_193_R, SMART_193_STEP, smart[193] & mask32);
-		if (afr < r)
-			afr = r;
-	}
-
 	if (smart[197] != SMART_UNASSIGNED) {
 		double r = smart_afr_value(SMART_197_R, SMART_197_STEP, smart[197] & mask32);
 		if (afr < r)
@@ -605,7 +535,7 @@ static double raid_prob_of_one_or_more_failures(double array_failure_rate, doubl
 	return poisson_prob_n_or_more_failures(raid_failure_rate, 1);
 }
 
-static void state_smart(unsigned n, tommy_list* low)
+static void state_smart(struct snapraid_state* state, unsigned n, tommy_list* low)
 {
 	tommy_node* i;
 	unsigned j;
@@ -665,6 +595,12 @@ static void state_smart(unsigned n, tommy_list* low)
 		double afr;
 		uint64_t flag;
 
+		/* clear attributes to ignore */
+		for (j = 0; j < SMART_IGNORE_MAX; ++j) {
+			devinfo->smart[state->smartignore[j]] = SMART_UNASSIGNED;
+			devinfo->smart[devinfo->smartignore[j]] = SMART_UNASSIGNED;
+		}
+
 		if (devinfo->smart[SMART_TEMPERATURE_CELSIUS] != SMART_UNASSIGNED)
 			printf("%7" PRIu64, devinfo->smart[SMART_TEMPERATURE_CELSIUS] & mask16);
 		else if (devinfo->smart[SMART_AIRFLOW_TEMPERATURE_CELSIUS] != SMART_UNASSIGNED)
@@ -831,62 +767,111 @@ bail:
 	}
 }
 
-/**
- * Fill with fake data the device list.
- */
-static int devtest(tommy_list* low, int operation)
+static void state_probe(struct snapraid_state* state, tommy_list* low)
 {
-	unsigned c;
+	tommy_node* i;
+	size_t device_pad;
 
-	if (operation != DEVICE_SMART)
+	(void)state;
+
+	/* compute lengths for padding */
+	device_pad = 0;
+	for (i = tommy_list_head(low); i != 0; i = i->next) {
+		size_t len;
+		devinfo_t* devinfo = i->data;
+
+		len = strlen(devinfo->file);
+		if (len > device_pad)
+			device_pad = len;
+	}
+
+	printf("SnapRAID PROBE report:\n");
+	printf("\n");
+	printf("   State ");
+	printf("  ");
+	printl("Device", device_pad);
+	printf("  Disk");
+	printf("\n");
+	/*      |<##################################################################72>|####80>| */
+	printf(" -----------------------------------------------------------------------\n");
+
+	for (i = tommy_list_head(low); i != 0; i = i->next) {
+		devinfo_t* devinfo = i->data;
+
+		printf("  ");
+
+		switch (devinfo->power) {
+		case POWER_STANDBY : printf("StandBy"); break;
+		case POWER_ACTIVE : printf(" Active"); break;
+		default : printf("Unknown"); break;
+		}
+
+		printf("  ");
+		if (*devinfo->file)
+			printl(devinfo->file, device_pad);
+		else
+			printl("-", device_pad);
+
+		printf("  ");
+		if (*devinfo->name)
+			printf("%s", devinfo->name);
+		else
+			printf("-");
+
+		printf("\n");
+
+		log_tag("probe:%s:%s:%d\n", devinfo->file, devinfo->name, devinfo->power);
+	}
+
+	printf("\n");
+}
+
+int devtest(tommy_list* high, tommy_list* low, int operation)
+{
+	tommy_node* i;
+	unsigned count;
+
+	if (operation != DEVICE_SMART && operation != DEVICE_PROBE)
 		return -1;
 
-	/* add some fake data */
-	for (c = 0; c < 16; ++c) {
+	/* for each device add some fake data */
+	count = 0;
+	for (i = tommy_list_head(high); i != 0; i = i->next) {
+		devinfo_t* devinfo = i->data;
 		devinfo_t* entry;
 		int j;
 
+		++count;
+
 		entry = calloc_nofail(1, sizeof(devinfo_t));
 
-		entry->device = c;
+		/* fake device number */
+		entry->device = tommy_strhash_u32(0, devinfo->name);
 
 		tommy_list_insert_tail(low, &entry->node, entry);
 
 		for (j = 0; j < 256; ++j) {
-			switch (c) {
+			switch (count) {
 			case 0 : entry->smart[j] = 0; break;
 			case 1 : entry->smart[j] = SMART_UNASSIGNED; break;
 			default :
-				if (j == 193)
-					entry->smart[j] = c - 2;
-				else
-					entry->smart[j] = 0;
+				entry->smart[j] = 0;
 				break;
 			}
 		}
 
-		if (c == 0) {
-			entry->smart_serial[0] = 0;
-			entry->smart_vendor[0] = 0;
-			entry->smart_model[0] = 0;
-			entry->file[0] = 0;
-			entry->name[0] = 0;
-			entry->smart[SMART_SIZE] = SMART_UNASSIGNED;
-			entry->smart[SMART_ROTATION_RATE] = 0;
-		} else {
-			snprintf(entry->smart_serial, sizeof(entry->smart_serial), "S%u", c);
-			snprintf(entry->smart_vendor, sizeof(entry->smart_vendor), "V%u", c);
-			snprintf(entry->smart_model, sizeof(entry->smart_model), "M%u", c);
-			pathcpy(entry->file, sizeof(entry->name), "file");
-			pathcpy(entry->name, sizeof(entry->name), "name");
-			entry->smart[SMART_SIZE] = c * TERA;
-			entry->smart[SMART_ROTATION_RATE] = 7200;
-		}
-
+		pathprint(entry->smart_serial, sizeof(entry->smart_serial), "FAKE_%s", devinfo->smart_serial);
+		pathprint(entry->smart_vendor, sizeof(entry->smart_vendor), "FAKE_%s", devinfo->smart_vendor);
+		pathprint(entry->smart_model, sizeof(entry->smart_model), "FAKE_%s", devinfo->smart_model);
+		pathprint(entry->file, sizeof(entry->name), "FAKE_%s", devinfo->file);
+		pathcpy(entry->name, sizeof(entry->name), devinfo->name);
+		entry->smart[SMART_SIZE] = count * TERA;
+		entry->smart[SMART_ROTATION_RATE] = 7200;
+		entry->smart[SMART_TEMPERATURE_CELSIUS] = 27;
 		entry->smart[SMART_ERROR] = 0;
 		entry->smart[SMART_FLAGS] = SMART_UNASSIGNED;
 
-		switch (c) {
+		switch (count) {
 		case 3 : entry->smart[SMART_ERROR] = 1; break;
 		case 4 : entry->smart[SMART_FLAGS] = SMARTCTL_FLAG_UNSUPPORTED; break;
 		case 5 : entry->smart[SMART_FLAGS] = SMARTCTL_FLAG_COMMAND; break;
@@ -897,6 +882,8 @@ static int devtest(tommy_list* low, int operation)
 		case 10 : entry->smart[SMART_FLAGS] = SMARTCTL_FLAG_ERROR; break;
 		case 11 : entry->smart[SMART_FLAGS] = SMARTCTL_FLAG_ERROR_LOGGED; break;
 		}
+
+		entry->power = POWER_ACTIVE;
 	}
 
 	return 0;
@@ -932,6 +919,7 @@ void state_device(struct snapraid_state* state, int operation, tommy_list* filte
 		pathcpy(entry->name, sizeof(entry->name), disk->name);
 		pathcpy(entry->mount, sizeof(entry->mount), disk->dir);
 		pathcpy(entry->smartctl, sizeof(entry->smartctl), disk->smartctl);
+		memcpy(entry->smartignore, disk->smartignore, sizeof(entry->smartignore));
 
 		tommy_list_insert_tail(&high, &entry->node, entry);
 	}
@@ -951,6 +939,7 @@ void state_device(struct snapraid_state* state, int operation, tommy_list* filte
 			pathcpy(entry->name, sizeof(entry->name), lev_config_name(j));
 			pathcpy(entry->mount, sizeof(entry->mount), state->parity[j].split_map[s].path);
 			pathcpy(entry->smartctl, sizeof(entry->smartctl), state->parity[j].smartctl);
+			memcpy(entry->smartignore, state->parity[j].smartignore, sizeof(entry->smartignore));
 			pathcut(entry->mount); /* remove the parity file */
 
 			tommy_list_insert_tail(&high, &entry->node, entry);
@@ -958,9 +947,9 @@ void state_device(struct snapraid_state* state, int operation, tommy_list* filte
 	}
 
 	if (state->opt.fake_device) {
-		ret = devtest(&low, operation);
+		ret = devtest(&high, &low, operation);
 	} else {
-		int others = operation == DEVICE_SMART;
+		int others = operation == DEVICE_SMART || operation == DEVICE_PROBE;
 
 		ret = devquery(&high, &low, operation, others);
 	}
@@ -976,6 +965,8 @@ void state_device(struct snapraid_state* state, int operation, tommy_list* filte
 		case DEVICE_DOWN : ope = "Spindown"; break;
 		case DEVICE_LIST : ope = "Device listing"; break;
 		case DEVICE_SMART : ope = "Smart"; break;
+		case DEVICE_PROBE : ope = "Probe"; break;
+		case DEVICE_DOWNIFUP : ope = "Spindown"; break;
 		}
 		log_fatal("%s is unsupported in this platform.\n", ope);
 	} else {
@@ -992,7 +983,10 @@ void state_device(struct snapraid_state* state, int operation, tommy_list* filte
 		}
 
 		if (operation == DEVICE_SMART)
-			state_smart(state->level + tommy_list_count(&state->disklist), &low);
+			state_smart(state, state->level + tommy_list_count(&state->disklist), &low);
+
+		if (operation == DEVICE_PROBE)
+			state_probe(state, &low);
 	}
 
 	tommy_list_foreach(&high, free);
