@@ -83,8 +83,7 @@ struct snapraid_task {
 /**
  * Worker for tasks.
  *
- * This represents a worker thread designated to read data
- * from a specific disk.
+ * This represents a worker thread designated to read/write data for a specific disk.
  */
 struct snapraid_worker {
 #if HAVE_THREAD
@@ -130,16 +129,77 @@ struct snapraid_worker {
 #define IO_WRITER_ERROR_MAX (-IO_WRITER_ERROR_BASE)
 
 /**
- * Reader.
+ * IO system that handle all parallel reads and writes.
  *
- * This represents the pool of worker threads dedicated to read
+ * This represents the pool of worker threads dedicated to read and write
  * data from the disks.
+ *
+ * Typycal scenario is:
+ *
+ * # Initialize the process
+ * io_start(&io, blockstart, blockmax, block_enabled);
+ * 
+ * for_each_block {
+ *         # Now read starts
+ *         # Get the current block to process and the vector of data buffers to use
+ *         #   buffer[0..diskmax-1] are data disks
+ *         #   buffer[diskmax..diskmax+levelmax-1] are parity disks
+ *         #
+ *         # INTERNAL: Broadcast a read_sched signal, to signal worker threads
+ *         # that the previous "blockcur" position is now available and the
+ *         # buffers can be used to read a new look-ahead position.
+ *         unsigned blockcur = io_read_next(&io, &buffer);
+ * 
+ *         for_each_disk {
+ *                 # Read a disk into buffer[diskcur]
+ *                 # diskcur is the disk processed, and it may came in random order
+ *                 # depending on which disk is faster
+ *                 #
+ *                 # INTERNAL: It may wait for the read_done condition in case
+ *                 # the read is not yet done.
+ *                 # The reader thread will set the condition when ready.
+ *                 struct snapraid_task* task = io_data_read(&io, &diskcur, waiting_map, &waiting_mac);
+ * 
+ *                 # compute the hash
+ *                 ...
+ *         }
+ *
+ *         # compute the parity
+ *         ...
+ *
+ *         # Now write starts
+ *         # At this point the parity data should ALREADY be in the buffer[diskmax+levcur]
+ *         # Note that the only reason that the process is separated for each parity
+ *         # is to allow to identify which is the latest and slowest disk
+ *         io_write_preset(&io, blockcur, !parity_going_to_be_updated);
+ *
+ *         for_each_parity {
+ *                 unsigned levcur;
+ *
+ *                 # Write a parity from buffer[diskmax+levcur]
+ *                 # levcur is the disk processed, and it may came in random order
+ *                 # depending on which disk is faster
+ *                 #
+ *                 # INTERNAL: It may wait for the write_done condition in case
+ *                 # there isn't a free position in the parity write queue.
+ *                 # The writer thread will set the condition when ready.
+ *                 io_parity_write(&io, &levcur, waiting_map, &waiting_mac);
+ *         }
+ *
+ *         # End of the write
+ *         # INTERNAL: Broadcast a write_sched signal, to signal worker threads
+ *         # that a new position is ready to be written
+ *         io_write_next(&io, blockcur, !parity_going_to_be_updated, writer_error);
+ * }
+ *
+ * # Stop all thread after writing all queued data
+ * io_stop(&io);
  */
 struct snapraid_io {
 	struct snapraid_state* state;
 
 	/**
-	 * Number of read-ahead buffers to use.
+	 * Number of read-ahead and write-cached buffers to use.
 	 *
 	 * Between IO_MIN and IO_MAX for thread use.
 	 *
@@ -257,13 +317,23 @@ struct snapraid_io {
 	int done;
 
 	/**
-	 * The task currently used by the caller.
+	 * The worker task index currently used by the operation in
+	 * progress. It's the position in all the reader task_map[].
+	 *
+	 * Until the operation is in progress, the data buffer cannot be
+	 * discarded, because it's in use.
 	 *
 	 * It's a rolling counter, when reaching ::io_max
 	 * it goes again to 0.
 	 *
-	 * When the caller finish with the current index,
-	 * it's incremented, and a read_sched() signal is sent.
+	 * When the caller finish with this index, it calls read_sched(),
+	 * and this ::reader_index is increamended, meaning that the data of
+	 * the just finished index is not needed anymore and the buffer is
+	 * available to be used for the following read-ahead.
+	 *
+	 * When this happens a read_sched signal is broadcasted, and all the
+	 * workers can proceed using this now available index to have one more
+	 * read-ahead buffer to load.
 	 *
 	 * In monothread mode it isn't the task index,
 	 * but the worker index.
@@ -271,13 +341,22 @@ struct snapraid_io {
 	unsigned reader_index;
 
 	/**
-	 * The task currently used by the caller.
+	 * The worker task index currently used by the operation in
+	 * progress. It's the position in all the writer task_map[].
+	 *
+	 * Until the operation is in progress, the data buffer cannot be
+	 * written, because the data is not yet in the buffer.
 	 *
 	 * It's a rolling counter, when reaching ::io_max
 	 * it goes again to 0.
 	 *
-	 * When the caller finish with the current index,
-	 * it's incremented, and a write_sched() signal is sent.
+	 * When the caller finish writing the data in the buffer for this
+	 * index, it calls write_sched() and ::writer_index is incremented,
+	 * meaning the the just finished index is scheduled to be written
+	 * to disk.
+	 *
+	 * When this happens a write_sched signal is broadcasted, and all the
+	 * workers can proceed writing the data in this index.
 	 *
 	 * In monothread mode it isn't the task index,
 	 * but the worker index.
@@ -288,6 +367,11 @@ struct snapraid_io {
 	 * Counts the error happening in the writers.
 	 */
 	int writer_error[IO_WRITER_ERROR_MAX];
+
+	/**
+	 * Bandwidth
+	 */
+	struct snapraid_bw bw;
 };
 
 /**
@@ -395,5 +479,10 @@ extern void (*io_write_next)(struct snapraid_io* io, block_off_t blockcur, int s
  */
 extern void (*io_refresh)(struct snapraid_io* io);
 
+/**
+ * Flush all the writes.
+ */
+extern void (*io_flush)(struct snapraid_io* io);
+
 #endif
 
