@@ -206,7 +206,7 @@ void os_abort(void)
 #ifdef __GNUC__
 	printf(", gcc " __VERSION__);
 #endif
-	printf(", %d-bit", (int)sizeof(void *) * 8);
+	printf(", %d-bit", (int)sizeof(void*) * 8);
 	printf(", PATH_MAX=%d", PATH_MAX);
 	printf("\n");
 
@@ -1047,6 +1047,157 @@ int windows_fsync(int fd)
 	return 0;
 }
 
+static int windows_vsync(const wchar_t* volume)
+{
+	HANDLE h;
+	DWORD bytes;
+
+	/* open the volume (volumeName already in \\?\Volume{GUID} format) */
+	h = CreateFileW(volume, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, NULL);
+	if (h == INVALID_HANDLE_VALUE) {
+		windows_errno(GetLastError());
+		return -1;
+	}
+
+	/*
+	 * "The FlushFileBuffers API can be used to flush all the outstanding data
+	 * and metadata on a single file or a whole volume. However, frequent use
+	 * of this API can cause reduced throughput. Internally, Windows uses the
+	 * SCSI Synchronize Cache or the IDE/ATAPI Flush cache commands."
+	 *
+	 * From:
+	 * "Windows Write Caching - Part 2 An overview for Application Developers"
+	 * http://winntfs.com/2012/11/29/windows-write-caching-part-2-an-overview-for-application-developers/
+	 */
+	if (!FlushFileBuffers(h)) {
+		DWORD error = GetLastError();
+		CloseHandle(h);
+
+		switch (error) {
+		case ERROR_INVALID_HANDLE :
+			/*
+			 * FlushFileBuffers returns this error if the handle
+			 * doesn't support buffering, like the console output.
+			 *
+			 * We had a report that also ATA-over-Ethernet returns
+			 * this error, but not enough sure to ignore it.
+			 * So, we use now an extended error reporting.
+			 */
+			log_fatal("Unexpected Windows INVALID_HANDLE error in FlushFileBuffers().\n");
+			log_fatal("Are you using ATA-over-Ethernet ? Please report it.\n");
+
+			/* normal error processing */
+			windows_errno(error);
+			return -1;
+
+		case ERROR_ACCESS_DENIED :
+			/*
+			 * FlushFileBuffers returns this error for read-only
+			 * data, that cannot have to be flushed.
+			 */
+			return 0;
+
+		default :
+			windows_errno(error);
+			return -1;
+		}
+	}
+
+	/* lock the volume (may fail if volume is in use, which is fine) */
+	if (!DeviceIoControl(h, FSCTL_LOCK_VOLUME, NULL, 0, NULL, 0, &bytes, NULL)) {
+		DWORD error = GetLastError();
+		CloseHandle(h);
+
+		switch (error) {
+		case ERROR_ACCESS_DENIED :
+			/*
+			 * ERROR_ACCESS_DENIED (5) means volume is in use - this is expected and OK
+			 */
+			return 0;
+
+		default :
+			windows_errno(error);
+			return -1;
+		}
+	}
+
+	/* unlock immediately - the lock/unlock cycle helps ensure consistency */
+	if (!DeviceIoControl(h, FSCTL_UNLOCK_VOLUME, NULL, 0, NULL, 0, &bytes, NULL)) {
+		DWORD error = GetLastError();
+		CloseHandle(h);
+		windows_errno(error);
+		return -1;
+	}
+
+	if (!CloseHandle(h)) {
+		windows_errno(GetLastError());
+		return -1;
+	}
+
+	return 0;
+}
+
+static int windows_async(void)
+{
+	HANDLE h;
+	wchar_t volume[MAX_PATH];
+	DWORD error = 0;
+	DWORD count = 0;
+	DWORD success = 0;
+
+	h = FindFirstVolumeW(volume, sizeof(volume) / sizeof(wchar_t));
+	if (h == INVALID_HANDLE_VALUE) {
+		windows_errno(GetLastError());
+		return -1;
+	}
+
+	do {
+		/* remove trailing backslash */
+		size_t len = wcslen(volume);
+		if (len > 0 && volume[len - 1] == L'\\')
+			volume[len - 1] = L'\0';
+
+		++count;
+
+		if (windows_vsync(volume) == 0) {
+			++success;
+		} else {
+			/* save the first error */
+			if (error == 0)
+				error = GetLastError();
+		}
+	} while (FindNextVolumeW(h, volume, sizeof(volume) / sizeof(wchar_t)));
+
+	/* if at least one failed */
+	if (count != success) {
+		FindVolumeClose(h);
+		windows_errno(error);
+		return -1;
+	}
+
+	error = GetLastError();
+	if (error != ERROR_NO_MORE_FILES) {
+		FindVolumeClose(h);
+		windows_errno(error);
+		return -1;
+	}
+
+	if (!FindVolumeClose(h)) {
+		windows_errno(GetLastError());
+		return -1;
+	}
+
+	return 0;
+}
+
+int windows_sync(void)
+{
+	if (windows_async() != 0)
+		return -1;
+
+	return 0;
+}
+
 int windows_futimens(int fd, struct windows_timespec tv[2])
 {
 	HANDLE h;
@@ -1162,7 +1313,7 @@ int windows_rename(const char* from, const char* to)
 	 * Is an atomic file rename (with overwrite) possible on Windows?
 	 * http://stackoverflow.com/questions/167414/is-an-atomic-file-rename-with-overwrite-possible-on-windows
 	 */
-	if (!MoveFileExW(convert(conv_buf_from, from), convert(conv_buf_to, to), MOVEFILE_REPLACE_EXISTING)) {
+	if (!MoveFileExW(convert(conv_buf_from, from), convert(conv_buf_to, to), MOVEFILE_REPLACE_EXISTING | MOVEFILE_WRITE_THROUGH)) {
 		windows_errno(GetLastError());
 		return -1;
 	}
@@ -1210,7 +1361,7 @@ int windows_open(const char* file, int flags, ...)
 	case O_RDWR :
 		access = GENERIC_READ | GENERIC_WRITE;
 		break;
-	default:
+	default :
 		errno = EINVAL;
 		return -1;
 	}
@@ -1234,7 +1385,7 @@ int windows_open(const char* file, int flags, ...)
 	case O_TRUNC :
 		create = TRUNCATE_EXISTING;
 		break;
-	default:
+	default :
 		errno = EINVAL;
 		return -1;
 	}
@@ -1549,9 +1700,21 @@ const char* windows_stat_desc(struct stat* st)
 	return st->st_desc;
 }
 
-void windows_sleep(unsigned seconds)
+unsigned windows_sleep(unsigned seconds)
 {
-	Sleep(seconds * 1000);
+	while (seconds) {
+		if (global_interrupt) /* SIGINT should stop the sleep */
+			break;
+		Sleep(1000);
+		--seconds;
+	}
+
+	return seconds;
+}
+
+void windows_usleep(uint64_t useconds)
+{
+	Sleep(useconds / 1000);
 }
 
 int windows_link(const char* existing, const char* file)
@@ -1693,8 +1856,10 @@ int windows_readlink(const char* file, char* buffer, size_t size)
 	return len;
 }
 
-int devuuid(uint64_t device, char* uuid, size_t uuid_size)
+int devuuid(uint64_t device, const char* path, char* uuid, size_t uuid_size)
 {
+	(void)path;
+
 	/* just use the volume serial number returned in the device parameter */
 	snprintf(uuid, uuid_size, "%08x", (unsigned)device);
 
@@ -2147,7 +2312,7 @@ static int devresolve(const char* mount, char* file, size_t file_size, char* wfi
 /**
  * Read a device tree filling the specified list of disk_t entries.
  */
-static int devtree(const char* name, const char* custom, const char* wfile, devinfo_t* parent, tommy_list* list)
+static int devtree(const char* name, const char* smartctl, const int* smartignore, const char* wfile, devinfo_t* parent, tommy_list* list)
 {
 	wchar_t conv_buf[CONV_MAX];
 	HANDLE h;
@@ -2196,7 +2361,8 @@ static int devtree(const char* name, const char* custom, const char* wfile, devi
 		devinfo = calloc_nofail(1, sizeof(devinfo_t));
 
 		pathcpy(devinfo->name, sizeof(devinfo->name), name);
-		pathcpy(devinfo->smartctl, sizeof(devinfo->smartctl), custom);
+		pathcpy(devinfo->smartctl, sizeof(devinfo->smartctl), smartctl);
+		memcpy(devinfo->smartignore, smartignore, sizeof(devinfo->smartignore));
 		devinfo->device = vde->Extents[i].DiskNumber;
 		pathprint(devinfo->file, sizeof(devinfo->file), "/dev/pd%" PRIu64, devinfo->device);
 		pathprint(devinfo->wfile, sizeof(devinfo->wfile), "\\\\.\\PhysicalDrive%" PRIu64, devinfo->device);
@@ -2332,7 +2498,7 @@ static int devscan(tommy_list* list)
 /**
  * Get SMART attributes.
  */
-static int devsmart(uint64_t device, const char* name, const char* custom, uint64_t* smart, char* serial, char* vendor, char* model)
+static int devsmart(uint64_t device, const char* name, const char* smartctl, uint64_t* smart, char* serial, char* vendor, char* model)
 {
 	char conv_buf[CONV_MAX];
 	WCHAR cmd[MAX_PATH + 128];
@@ -2343,10 +2509,10 @@ static int devsmart(uint64_t device, const char* name, const char* custom, uint6
 
 	snprintf(file, sizeof(file), "/dev/pd%" PRIu64, device);
 
-	/* if there is a custom command */
-	if (custom[0]) {
+	/* if there is a custom smartctl command */
+	if (smartctl[0]) {
 		char option[128];
-		snprintf(option, sizeof(option), custom, file);
+		snprintf(option, sizeof(option), smartctl, file);
 		snwprintf(cmd, sizeof(cmd), L"\"%lssmartctl.exe\" -a %s", exedir, option);
 	} else {
 		snwprintf(cmd, sizeof(cmd), L"\"%lssmartctl.exe\" -a %s", exedir, file);
@@ -2383,8 +2549,8 @@ retry:
 		/* LCOV_EXCL_STOP */
 	}
 
-	/* if first try without custom command */
-	if (count == 0 && custom[0] == 0) {
+	/* if first try without custom smartctl command */
+	if (count == 0 && smartctl[0] == 0) {
 		/*
 		 * Handle some common cases in Windows.
 		 *
@@ -2416,9 +2582,9 @@ retry:
 }
 
 /**
- * Spin down a specific device.
+ * Get POWER state
  */
-static int devdown(uint64_t device, const char* name, const char* custom)
+static int devprobe(uint64_t device, const char* name, const char* smartctl, int* power)
 {
 	char conv_buf[CONV_MAX];
 	WCHAR cmd[MAX_PATH + 128];
@@ -2429,10 +2595,93 @@ static int devdown(uint64_t device, const char* name, const char* custom)
 
 	snprintf(file, sizeof(file), "/dev/pd%" PRIu64, device);
 
-	/* if there is a custom command */
-	if (custom[0]) {
+	/* if there is a custom smartctl command */
+	if (smartctl[0]) {
 		char option[128];
-		snprintf(option, sizeof(option), custom, file);
+		snprintf(option, sizeof(option), smartctl, file);
+		snwprintf(cmd, sizeof(cmd), L"\"%lssmartctl.exe\" -n standby,3 -i %s", exedir, option);
+	} else {
+		snwprintf(cmd, sizeof(cmd), L"\"%lssmartctl.exe\" -n standby,3 -i %s", exedir, file);
+	}
+
+	count = 0;
+
+retry:
+	log_tag("smartctl:%s:%s:run: %s\n", file, name, u16tou8(conv_buf, cmd));
+
+	f = _wpopen(cmd, L"rt");
+	if (!f) {
+		/* LCOV_EXCL_START */
+		log_fatal("Failed to run '%s' (from popen).\n", u16tou8(conv_buf, cmd));
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	if (smartctl_flush(f, file, name) != 0) {
+		/* LCOV_EXCL_START */
+		pclose(f);
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	ret = pclose(f);
+
+	log_tag("smartctl:%s:%s:ret: %x\n", file, name, ret);
+
+	if (ret == -1) {
+		/* LCOV_EXCL_START */
+		log_fatal("Failed to run '%s' (from pclose).\n", u16tou8(conv_buf, cmd));
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	/* if first try without custom smartctl command */
+	if (count == 0 && smartctl[0] == 0) {
+		/*
+		 * Handle some common cases in Windows.
+		 *
+		 * Sometimes the "type" autodetection is wrong, and the command fails at identification
+		 * stage, returning with error 2.
+		 *
+		 * In such conditions we retry using the "sat" type, that often allows to proceed.
+		 */
+		if (ret == 2) {
+			/* retry using the "sat" type */
+			snwprintf(cmd, sizeof(cmd), L"\"%lssmartctl.exe\" -a -d sat %s", exedir, file);
+
+			++count;
+			goto retry;
+		}
+	}
+
+	if (ret == 0)
+		*power = POWER_ACTIVE;
+	else if (ret == 3)
+		*power = POWER_STANDBY;
+	else
+		*power = POWER_UNKNOWN;
+
+	return 0;
+}
+
+/**
+ * Spin down a specific device.
+ */
+static int devdown(uint64_t device, const char* name, const char* smartctl)
+{
+	char conv_buf[CONV_MAX];
+	WCHAR cmd[MAX_PATH + 128];
+	char file[128];
+	FILE* f;
+	int ret;
+	int count;
+
+	snprintf(file, sizeof(file), "/dev/pd%" PRIu64, device);
+
+	/* if there is a custom smartctl command */
+	if (smartctl[0]) {
+		char option[128];
+		snprintf(option, sizeof(option), smartctl, file);
 		snwprintf(cmd, sizeof(cmd), L"\"%lssmartctl.exe\" -s standby,now %s", exedir, option);
 	} else {
 		snwprintf(cmd, sizeof(cmd), L"\"%lssmartctl.exe\" -s standby,now %s", exedir, file);
@@ -2469,8 +2718,8 @@ retry:
 		/* LCOV_EXCL_STOP */
 	}
 
-	/* if first try without custom command */
-	if (count == 0 && custom[0] == 0) {
+	/* if first try without custom smartctl command */
+	if (count == 0 && smartctl[0] == 0) {
 		/*
 		 * Handle some common cases in Windows.
 		 *
@@ -2499,63 +2748,88 @@ retry:
 }
 
 /**
- * Spin up a device.
- *
- * There isn't a defined way to spin up a device,
- * so we just do a generic write.
+ * Spin down a specific device if it's up
  */
-static int devup(const char* mount)
+static int devdownifup(uint64_t device, const char* name, const char* smartctl, int* power)
+{
+	*power = POWER_UNKNOWN;
+
+	if (devprobe(device, name, smartctl, power) != 0)
+		return -1;
+
+	if (*power == POWER_ACTIVE)
+		return devdown(device, name, smartctl);
+
+	return 0;
+}
+
+/**
+ * Spin up a device.
+ */
+static int devup(const char* wfile)
 {
 	wchar_t conv_buf[CONV_MAX];
-	int f;
-	char path[PATH_MAX];
+	HANDLE h;
+	BOOL ret;
+	DISK_GEOMETRY dg;
+	DWORD bytes;
+	void* buffer;
 
-	/* add a temporary name used for writing */
-	pathprint(path, sizeof(path), "%s.snapraid-spinup.tmp", mount);
+	/* open the volume */
+	h = CreateFileW(convert(conv_buf, wfile), GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, 0, OPEN_EXISTING, FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH, 0);
+	if (h == INVALID_HANDLE_VALUE) {
+		windows_errno(GetLastError());
+		return -1;
+	}
 
-	/* create a temporary file, automatically deleted on close */
-	f = _wopen(convert(conv_buf, path), _O_CREAT | _O_TEMPORARY | _O_RDWR, _S_IREAD | _S_IWRITE);
-	if (f != -1)
-		close(f);
+	bytes = 0;
+	ret = DeviceIoControl(h, IOCTL_DISK_GET_DRIVE_GEOMETRY, NULL, 0, &dg, sizeof(dg), &bytes, NULL);
+	if (!ret) {
+		DWORD error = GetLastError();
+		CloseHandle(h);
+		windows_errno(error);
+		return -1;
+	}
+
+	buffer = VirtualAlloc(NULL, dg.BytesPerSector, MEM_COMMIT | MEM_RESERVE, PAGE_READWRITE);
+	if (!buffer) {
+		DWORD error = GetLastError();
+		CloseHandle(h);
+		windows_errno(error);
+		return -1;
+	}
+
+	bytes = 0;
+	if (!ReadFile(h, buffer, dg.BytesPerSector, &bytes, NULL)) {
+		DWORD error = GetLastError();
+		CloseHandle(h);
+		windows_errno(error);
+		return -1;
+	}
+
+	if (!CloseHandle(h)) {
+		DWORD error = GetLastError();
+		VirtualFree(buffer, 0, MEM_RELEASE);
+		windows_errno(error);
+		return -1;
+	}
+
+	VirtualFree(buffer, 0, MEM_RELEASE);
 
 	return 0;
 }
 
 /**
  * Thread for spinning up.
- *
- * Note that filling up the devinfo object is done inside this thread,
- * to avoid to block the main thread if the device need to be spin up
- * to handle stat/resolve requests.
  */
 static void* thread_spinup(void* arg)
 {
 	devinfo_t* devinfo = arg;
-	struct stat st;
 	uint64_t start;
 
 	start = tick_ms();
 
-	/* uses lstat_sync() that maps to CreateFile */
-	/* we cannot use FindFirstFile because it doesn't allow to open the root dir */
-	if (lstat_sync(devinfo->mount, &st, 0) != 0) {
-		/* LCOV_EXCL_START */
-		log_fatal("Failed to stat path '%s'. %s.\n", devinfo->mount, strerror(errno));
-		return (void*)-1;
-		/* LCOV_EXCL_STOP */
-	}
-
-	/* set the device number */
-	devinfo->device = st.st_dev;
-
-	if (devresolve(devinfo->mount, devinfo->file, sizeof(devinfo->file), devinfo->wfile, sizeof(devinfo->wfile)) != 0) {
-		/* LCOV_EXCL_START */
-		log_fatal("Failed to resolve path '%s'.\n", devinfo->mount);
-		return (void*)-1;
-		/* LCOV_EXCL_STOP */
-	}
-
-	if (devup(devinfo->mount) != 0) {
+	if (devup(devinfo->wfile) != 0) {
 		/* LCOV_EXCL_START */
 		return (void*)-1;
 		/* LCOV_EXCL_STOP */
@@ -2587,6 +2861,29 @@ static void* thread_spindown(void* arg)
 	return 0;
 }
 
+/**
+ * Thread for spinning down.
+ */
+static void* thread_spindownifup(void* arg)
+{
+	devinfo_t* devinfo = arg;
+	uint64_t start;
+	int power;
+
+	start = tick_ms();
+
+	if (devdownifup(devinfo->device, devinfo->name, devinfo->smartctl, &power) != 0) {
+		/* LCOV_EXCL_START */
+		return (void*)-1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	if (power == POWER_ACTIVE)
+		msg_status("Spundown device '%s' for disk '%s' in %" PRIu64 " ms.\n", devinfo->file, devinfo->name, tick_ms() - start);
+
+	return 0;
+}
+
 /**
  * Thread for getting smart info.
  */
@@ -2603,6 +2900,22 @@ static void* thread_smart(void* arg)
 	return 0;
 }
 
+/**
+ * Thread for getting power info.
+ */
+static void* thread_probe(void* arg)
+{
+	devinfo_t* devinfo = arg;
+
+	if (devprobe(devinfo->device, devinfo->name, devinfo->smartctl, &devinfo->power) != 0) {
+		/* LCOV_EXCL_START */
+		return (void*)-1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	return 0;
+}
+
 static int device_thread(tommy_list* list, void* (*func)(void* arg))
 {
 	int fail = 0;
@@ -2640,42 +2953,23 @@ int devquery(tommy_list* high, tommy_list* low, int operation, int others)
 	tommy_node* i;
 	void* (*func)(void* arg) = 0;
 
-	if (operation != DEVICE_UP) {
-		/* for each device */
-		for (i = tommy_list_head(high); i != 0; i = i->next) {
-			devinfo_t* devinfo = i->data;
+	/* for each device */
+	for (i = tommy_list_head(high); i != 0; i = i->next) {
+		devinfo_t* devinfo = i->data;
 
-			if (devresolve(devinfo->mount, devinfo->file, sizeof(devinfo->file), devinfo->wfile, sizeof(devinfo->wfile)) != 0) {
-				/* LCOV_EXCL_START */
-				log_fatal("Failed to resolve path '%s'.\n", devinfo->mount);
-				return -1;
-				/* LCOV_EXCL_STOP */
-			}
-
-			/* expand the tree of devices */
-			if (devtree(devinfo->name, devinfo->smartctl, devinfo->wfile, devinfo, low) != 0) {
-				/* LCOV_EXCL_START */
-				log_fatal("Failed to expand device '%s'.\n", devinfo->file);
-				return -1;
-				/* LCOV_EXCL_STOP */
-			}
+		if (devresolve(devinfo->mount, devinfo->file, sizeof(devinfo->file), devinfo->wfile, sizeof(devinfo->wfile)) != 0) {
+			/* LCOV_EXCL_START */
+			log_fatal("Failed to resolve path '%s'.\n", devinfo->mount);
+			return -1;
+			/* LCOV_EXCL_STOP */
 		}
-	}
 
-	if (operation == DEVICE_UP) {
-		/* duplicate the high */
-		for (i = tommy_list_head(high); i != 0; i = i->next) {
-			devinfo_t* devinfo = i->data;
-			devinfo_t* entry;
-
-			entry = calloc_nofail(1, sizeof(devinfo_t));
-
-			entry->device = devinfo->device;
-			pathcpy(entry->name, sizeof(entry->name), devinfo->name);
-			pathcpy(entry->mount, sizeof(entry->mount), devinfo->mount);
-
-			/* insert in the high */
-			tommy_list_insert_tail(low, &entry->node, entry);
+		/* expand the tree of devices */
+		if (devtree(devinfo->name, devinfo->smartctl, devinfo->smartignore, devinfo->wfile, devinfo, low) != 0) {
+			/* LCOV_EXCL_START */
+			log_fatal("Failed to expand device '%s'.\n", devinfo->file);
+			return -1;
+			/* LCOV_EXCL_STOP */
 		}
 	}
 
@@ -2693,6 +2987,8 @@ int devquery(tommy_list* high, tommy_list* low, int operation, int others)
 	case DEVICE_UP : func = thread_spinup; break;
 	case DEVICE_DOWN : func = thread_spindown; break;
 	case DEVICE_SMART : func = thread_smart; break;
+	case DEVICE_PROBE : func = thread_probe; break;
+	case DEVICE_DOWNIFUP : func = thread_spindownifup; break;
 	}
 
 	if (!func)
@@ -2774,7 +3070,7 @@ int windows_cond_wait(windows_cond_t* cond, windows_mutex_t* mutex)
 }
 
 struct windows_key_context {
-	void (* func)(void *);
+	void (*func)(void*);
 	DWORD key;
 	tommy_node node;
 };
@@ -2782,7 +3078,7 @@ struct windows_key_context {
 /* list of all keys with destructor */
 static tommy_list windows_key_list = { 0 };
 
-int windows_key_create(windows_key_t* key, void(* destructor)(void*))
+int windows_key_create(windows_key_t* key, void (*destructor)(void*))
 {
 	struct windows_key_context* context;
 
@@ -2844,7 +3140,7 @@ int windows_setspecific(windows_key_t key, void* value)
 struct windows_thread_context {
 	HANDLE h;
 	unsigned id;
-	void* (* func)(void *);
+	void* (*func)(void*);
 	void* arg;
 	void* ret;
 };
@@ -2872,7 +3168,7 @@ static unsigned __stdcall windows_thread_func(void* arg)
 	return 0;
 }
 
-int windows_create(thread_id_t* thread, void* attr, void* (* func)(void *), void* arg)
+int windows_create(thread_id_t* thread, void* attr, void* (*func)(void*), void* arg)
 {
 	struct windows_thread_context* context;
 
@@ -2918,5 +3214,10 @@ int windows_join(thread_id_t thread, void** retval)
 	return 0;
 }
 
+int ambient_temperature(void)
+{
+	return 0;
+}
+
 #endif
 
