@@ -19,6 +19,9 @@
 #define __PARITY_H
 
 #include "support.h"
+#include "bw.h"
+#include "state.h"
+#include "io.h"
 
 /****************************************************************************/
 /* parity */
@@ -57,6 +60,7 @@ struct snapraid_parity_handle {
 	struct snapraid_split_handle split_map[SPLIT_MAX];
 	unsigned split_mac; /**< Number of parity splits. */
 	unsigned level; /**< Level of the parity. */
+	struct snapraid_bw* bw; /**< Context for bandwidth limiting. */
 };
 
 /**
@@ -143,5 +147,11 @@ int parity_read(struct snapraid_parity_handle* handle, block_off_t pos, unsigned
  */
 int parity_write(struct snapraid_parity_handle* handle, block_off_t pos, unsigned char* block_buffer, unsigned block_size);
 
+/**
+ * Flush and sync the parity files.
+ */
+int state_flush(struct snapraid_state* state, struct snapraid_io* io, struct snapraid_parity_handle* parity_handle, block_off_t blockcur);
+
+
 #endif
 
