@@ -238,6 +238,11 @@
 #include <execinfo.h>
 #endif
 
+#ifdef __APPLE__
+#include <CoreFoundation/CoreFoundation.h>
+#include <DiskArbitration/DiskArbitration.h>
+#endif
+
 /**
  * Enable thread use.
  */
@@ -338,7 +343,7 @@ static inline int hardlink(const char* a, const char* b)
  * Get the device UUID.
  * Return 0 on success.
  */
-int devuuid(uint64_t device, char* uuid, size_t size);
+int devuuid(uint64_t device, const char* path, char* uuid, size_t size);
 
 /**
  * Physical offset not yet read.
@@ -440,7 +445,6 @@ int randomize(void* ptr, size_t size);
 #define SMART_REALLOCATED_SECTOR_COUNT 5
 #define SMART_POWER_ON_HOURS 9
 #define SMART_AIRFLOW_TEMPERATURE_CELSIUS 190
-#define SMART_LOAD_CYCLE_COUNT 193
 #define SMART_TEMPERATURE_CELSIUS 194
 
 /**
@@ -456,6 +460,8 @@ int randomize(void* ptr, size_t size);
  */
 #define SMART_COUNT 260
 
+#define SMART_IGNORE_MAX 4 /**< Max number of ignored smart attributes */
+
 /**
  * Flags returned by smartctl.
  */
@@ -478,6 +484,14 @@ int randomize(void* ptr, size_t size);
  */
 #define SMART_UNASSIGNED 0xFFFFFFFFFFFFFFFFULL
 
+
+/**
+ * Power mode
+ */
+#define POWER_STANDBY 0
+#define POWER_ACTIVE 1
+#define POWER_UNKNOWN -1
+
 /**
  * Device info entry.
  */
@@ -486,6 +500,7 @@ struct devinfo_struct {
 	char name[PATH_MAX]; /**< Name of the disk. */
 	char mount[PATH_MAX]; /**< Mount point or other contained directory. */
 	char smartctl[PATH_MAX]; /**< Options for smartctl. */
+	int smartignore[SMART_IGNORE_MAX]; /**< Attribues to ignore */
 	char file[PATH_MAX]; /**< File device. */
 #ifdef _WIN32
 	char wfile[PATH_MAX]; /**< File device in Windows format. Like \\.\PhysicalDriveX, or \\?\Volume{X}. */
@@ -495,6 +510,7 @@ struct devinfo_struct {
 	char smart_serial[SMART_MAX]; /**< SMART serial number. */
 	char smart_vendor[SMART_MAX]; /**< SMART vendor. */
 	char smart_model[SMART_MAX]; /**< SMART model. */
+	int power; /**< POWER mode. */
 #if HAVE_THREAD
 	thread_id_t thread;
 #endif
@@ -506,6 +522,8 @@ typedef struct devinfo_struct devinfo_t;
 #define DEVICE_DOWN 1
 #define DEVICE_UP 2
 #define DEVICE_SMART 3
+#define DEVICE_PROBE 4
+#define DEVICE_DOWNIFUP 5
 
 /**
  * Query all the "high" level devices with the specified operation,
@@ -515,5 +533,22 @@ typedef struct devinfo_struct devinfo_t;
  */
 int devquery(tommy_list* high, tommy_list* low, int operation, int others);
 
+/**
+ * Fill with fake data the device list.
+ */
+int devtest(tommy_list* high, tommy_list* low, int operation);
+
+/**
+ * Get the ambient temperature in degree
+ *
+ * Return 0 if not available.
+ */
+int ambient_temperature(void);
+
+/**
+ * Global variable to identify if Ctrl+C is pressed.
+ */
+extern volatile int global_interrupt;
+
 #endif
 
