@@ -320,7 +320,9 @@ static int state_scrub_process(struct snapraid_state* state, struct snapraid_par
 	/* start all the worker threads */
 	io_start(&io, blockstart, blockmax, block_enabled);
 
-	state_progress_begin(state, blockstart, blockmax, countmax);
+	if (!state_progress_begin(state, blockstart, blockmax, countmax))
+		goto end;
+
 	while (1) {
 		unsigned char* buffer_recov[LEV_MAX];
 		snapraid_info info;
@@ -625,6 +627,21 @@ static int state_scrub_process(struct snapraid_state* state, struct snapraid_par
 			/* LCOV_EXCL_STOP */
 		}
 
+		/* thermal control */
+		if (state_thermal_alarm(state)) {
+			/* until now is misc */
+			state_usage_misc(state);
+
+			state_progress_stop(state);
+
+			state_thermal_cooldown(state);
+
+			state_progress_restart(state);
+
+			/* drop until now */
+			state_usage_waste(state);
+		}
+
 		/* autosave */
 		if (state->autosave != 0
 			&& autosavedone >= autosavelimit /* if we have reached the limit */
@@ -647,7 +664,12 @@ static int state_scrub_process(struct snapraid_state* state, struct snapraid_par
 		}
 	}
 
-	state_progress_end(state, countpos, countmax, countsize);
+end:
+	state_progress_end(state, countpos, countmax, countsize, "Nothing to scrub. Use the -p PLAN option to select a different plan, like -p full.\n");
+
+	/* save the new state if required */
+	if (state->need_write || state->opt.force_content_write)
+		state_write(state);
 
 	state_usage_print(state);
 
@@ -770,10 +792,13 @@ int state_scrub(struct snapraid_state* state, int plan, int olderthan)
 		ps.plan = SCRUB_EVEN;
 	} else if (plan == SCRUB_FULL) {
 		ps.plan = SCRUB_FULL;
+		msg_info("Scrub plan: full. All data blocks will be checked.\n");
 	} else if (plan == SCRUB_NEW) {
 		ps.plan = SCRUB_NEW;
+		msg_info("Scrub plan: new. Only blocks that have never been scrubbed will be checked.\n");
 	} else if (plan == SCRUB_BAD) {
 		ps.plan = SCRUB_BAD;
+		msg_info("Scrub plan: bad. Only blocks previously marked as bad will be checked.\n");
 	} else if (state->opt.force_scrub_at) {
 		/* scrub the specified amount of blocks */
 		ps.plan = SCRUB_AUTO;
@@ -794,6 +819,18 @@ int state_scrub(struct snapraid_state* state, int plan, int olderthan)
 			/* by default use a 10 day time limit */
 			recentlimit = now - 10 * 24 * 3600;
 		}
+
+		if (plan >= 0) {
+			if (olderthan >= 0)
+				msg_info("Scrub plan: auto. %d%% of the array, older than %d days, will be checked.\n", plan, olderthan);
+			else
+				msg_info("Scrub plan: auto. %d%% of the array, older than 10 days, will be checked.\n", plan);
+		} else {
+			if (olderthan >= 0)
+				msg_info("Scrub plan: auto. 8.3%% of the array, older than %d days, will be checked.\n", olderthan);
+			else
+				msg_info("Scrub plan: auto. 8.3%% of the array, older than 10 days, will be checked.\n");
+		}
 	}
 
 	/* identify the time limit */
