@@ -74,6 +74,7 @@ void usage(void)
 	printf("  " SWITCH_GETOPT_LONG("-N, --force-nocopy    ", "-N") "  Force commands disabling the copy detection\n");
 	printf("  " SWITCH_GETOPT_LONG("-F, --force-full      ", "-F") "  Force a full parity computation in sync\n");
 	printf("  " SWITCH_GETOPT_LONG("-R, --force-realloc   ", "-R") "  Force a full parity reallocation in sync\n");
+	printf("  " SWITCH_GETOPT_LONG("-w, --bw-limit RATE   ", "-w") "  Limit IO bandwidth (M|G)\n");
 	printf("  " SWITCH_GETOPT_LONG("-v, --verbose         ", "-v") "  Verbose\n");
 }
 
@@ -100,6 +101,29 @@ void test(int argc, char* argv[])
 	if (argc < 2 || strcmp(argv[1], "test") != 0)
 		return;
 
+	assert(strcmp(strpolish(strcpy(buffer, "\r \n\xFF")), "    ") == 0);
+	assert(strcmp(strtrim(strcpy(buffer, " trim trim \n\r")), "trim trim") == 0);
+	assert(strcmp(strlwr(strcpy(buffer, " LoWer\n\r")), " lower\n\r") == 0);
+
+	assert(worddigitstr("longneedlestring","needle") == 0);
+	assert(worddigitstr("longneedlestring","") == 0);
+	assert(worddigitstr("long needle string","needle") != 0);
+	assert(worddigitstr("long1needle2string","needle") != 0);
+	assert(worddigitstr("long\rneedle3string","needle") != 0);
+	assert(worddigitstr("long1needle","needle") != 0);
+	assert(worddigitstr("needle2string","needle") != 0);
+	assert(worddigitstr("needle","needle") != 0);
+
+	assert(strcmp(esc_tag("simple", buffer), "simple") == 0);
+	assert(strcmp(esc_tag("line1\nline2", buffer), "line1\\nline2") == 0);
+	assert(strcmp(esc_tag("line1\rline2", buffer), "line1\\rline2") == 0);
+	assert(strcmp(esc_tag("key:value", buffer), "key\\dvalue") == 0);
+	assert(strcmp(esc_tag("C:\\path\\file", buffer), "C\\d\\\\path\\\\file") == 0);
+	assert(strcmp(esc_tag("A\nB\rC:D\\E", buffer), "A\\nB\\rC\\dD\\\\E") == 0);
+	assert(strcmp(esc_tag("endwith\\", buffer), "endwith\\\\") == 0);
+	assert(strcmp(esc_tag("", buffer), "") == 0);
+	assert(strcmp(esc_tag("\n\r:\\\\", buffer), "\\n\\r\\d\\\\\\\\") == 0);
+
 	for (i = 2; i < argc; ++i) {
 		printf("argv[%d]\n", i);
 		printf("\t#%s#\n", argv[i]);
@@ -107,13 +131,355 @@ void test(int argc, char* argv[])
 	}
 
 #ifdef _WIN32
+	/* basic cases - no special characters, no quotes needed */
+	assert(strcmp(esc_shell("simple", buffer), "simple") == 0);
+	assert(strcmp(esc_shell("file.txt", buffer), "file.txt") == 0);
+	assert(strcmp(esc_shell("file123", buffer), "file123") == 0);
+	assert(strcmp(esc_shell("file_name-test.doc", buffer), "file_name-test.doc") == 0);
+	assert(strcmp(esc_shell(",._+:@/-", buffer), ",._+:@/-") == 0);
+	assert(strcmp(esc_shell("C:\\Users\\test", buffer), "C:\\Users\\test") == 0);
+
+	/* space - requires quoting */
 	assert(strcmp(esc_shell(" ", buffer), "\" \"") == 0);
-	assert(strcmp(esc_shell(" \" ", buffer), "\" \"\\\"\" \"") == 0);
-	assert(strcmp(esc_shell("&|()<>^", buffer), "^&^|^(^)^<^>^^") == 0);
-	assert(strcmp(esc_shell("&|()<>^ ", buffer), "\"&|()<>^ \"") == 0);
+	assert(strcmp(esc_shell("file name.txt", buffer), "\"file name.txt\"") == 0);
+	assert(strcmp(esc_shell("my document.doc", buffer), "\"my document.doc\"") == 0);
+	assert(strcmp(esc_shell("  multiple  spaces  ", buffer), "\"  multiple  spaces  \"") == 0);
+
+	/* tab - requires quoting */
+	assert(strcmp(esc_shell("\t", buffer), "\"\t\"") == 0);
+	assert(strcmp(esc_shell("file\tname", buffer), "\"file\tname\"") == 0);
+
+	/* newline - requires quoting */
+	assert(strcmp(esc_shell("\n", buffer), "\"\n\"") == 0);
+	assert(strcmp(esc_shell("line1\nline2", buffer), "\"line1\nline2\"") == 0);
+
+	/* carriage return - requires quoting */
+	assert(strcmp(esc_shell("\r", buffer), "\"\r\"") == 0);
+	assert(strcmp(esc_shell("text\r\n", buffer), "\"text\r\n\"") == 0);
+
+	/* double quote - requires quoting and escaping with backslash */
+	assert(strcmp(esc_shell("\"", buffer), "\"\\\"\"") == 0);
+	assert(strcmp(esc_shell("file\"name", buffer), "\"file\\\"name\"") == 0);
+	assert(strcmp(esc_shell("\"quoted\"", buffer), "\"\\\"quoted\\\"\"") == 0);
+	assert(strcmp(esc_shell("say \"hello\"", buffer), "\"say \\\"hello\\\"\"") == 0);
+
+	/* ampersand - requires quoting */
+	assert(strcmp(esc_shell("&", buffer), "\"&\"") == 0);
+	assert(strcmp(esc_shell("file&name", buffer), "\"file&name\"") == 0);
+	assert(strcmp(esc_shell("a&b&c", buffer), "\"a&b&c\"") == 0);
+	assert(strcmp(esc_shell("file & name", buffer), "\"file & name\"") == 0);
+
+	/* pipe - requires quoting */
+	assert(strcmp(esc_shell("|", buffer), "\"|\"") == 0);
+	assert(strcmp(esc_shell("file|name", buffer), "\"file|name\"") == 0);
+	assert(strcmp(esc_shell("a | b", buffer), "\"a | b\"") == 0);
+
+	/* parentheses - requires quoting */
+	assert(strcmp(esc_shell("(", buffer), "\"(\"") == 0);
+	assert(strcmp(esc_shell(")", buffer), "\")\"") == 0);
+	assert(strcmp(esc_shell("(test)", buffer), "\"(test)\"") == 0);
+	assert(strcmp(esc_shell("file (1)", buffer), "\"file (1)\"") == 0);
+	assert(strcmp(esc_shell("file(copy)", buffer), "\"file(copy)\"") == 0);
+
+	/* angle brackets - requires quoting */
+	assert(strcmp(esc_shell("<", buffer), "\"<\"") == 0);
+	assert(strcmp(esc_shell(">", buffer), "\">\"") == 0);
+	assert(strcmp(esc_shell("a<b>c", buffer), "\"a<b>c\"") == 0);
+	assert(strcmp(esc_shell("file > output", buffer), "\"file > output\"") == 0);
+
+	/* caret - requires quoting */
+	assert(strcmp(esc_shell("^", buffer), "\"^\"") == 0);
+	assert(strcmp(esc_shell("test^test", buffer), "\"test^test\"") == 0);
+	assert(strcmp(esc_shell("a ^ b", buffer), "\"a ^ b\"") == 0);
+
+	/* multiple special chars - requires quoting */
+	assert(strcmp(esc_shell("&|()<>^", buffer), "\"&|()<>^\"") == 0);
+	assert(strcmp(esc_shell("test&|test", buffer), "\"test&|test\"") == 0);
+	assert(strcmp(esc_shell("a & b | c", buffer), "\"a & b | c\"") == 0);
+
+	/* percent sign - requires quoting */
+	assert(strcmp(esc_shell("%", buffer), "\"%\"") == 0);
+	assert(strcmp(esc_shell("%%", buffer), "\"%%\"") == 0);
+	assert(strcmp(esc_shell("%PATH%", buffer), "\"%PATH%\"") == 0);
+	assert(strcmp(esc_shell("test%var%test", buffer), "\"test%var%test\"") == 0);
+	assert(strcmp(esc_shell("%PATH% file", buffer), "\"%PATH% file\"") == 0);
+
+	/* exclamation mark - requires quoting */
+	assert(strcmp(esc_shell("!", buffer), "\"!\"") == 0);
+	assert(strcmp(esc_shell("!VAR!", buffer), "\"!VAR!\"") == 0);
+	assert(strcmp(esc_shell("test!test", buffer), "\"test!test\"") == 0);
+	assert(strcmp(esc_shell("hello !world!", buffer), "\"hello !world!\"") == 0);
+
+	/* equals sign - requires quoting */
+	assert(strcmp(esc_shell("=", buffer), "\"=\"") == 0);
+	assert(strcmp(esc_shell("VAR=value", buffer), "\"VAR=value\"") == 0);
+	assert(strcmp(esc_shell("a=b", buffer), "\"a=b\"") == 0);
+
+	/* semicolon - requires quoting */
+	assert(strcmp(esc_shell(";", buffer), "\";\"") == 0);
+	assert(strcmp(esc_shell("cmd1;cmd2", buffer), "\"cmd1;cmd2\"") == 0);
+
+	/* backslash - no quotes needed when alone or in path */
+	assert(strcmp(esc_shell("\\", buffer), "\\") == 0);
+	assert(strcmp(esc_shell("C:\\", buffer), "C:\\") == 0);
+	assert(strcmp(esc_shell("C:\\Users", buffer), "C:\\Users") == 0);
+	assert(strcmp(esc_shell("path\\to\\file", buffer), "path\\to\\file") == 0);
+	assert(strcmp(esc_shell("C:\\folder\\", buffer), "C:\\folder\\") == 0);
+
+	/* backslash with space - requires quoting, normal backslash inside */
+	assert(strcmp(esc_shell("\\ ", buffer), "\"\\ \"") == 0);
+	assert(strcmp(esc_shell("C:\\ ", buffer), "\"C:\\ \"") == 0);
+	assert(strcmp(esc_shell("C:\\Program Files", buffer), "\"C:\\Program Files\"") == 0);
+
+	/* trailing backslash with quotes - backslashes before closing quote must be doubled */
+	assert(strcmp(esc_shell("C:\\folder\\ ", buffer), "\"C:\\folder\\ \"") == 0);
+	assert(strcmp(esc_shell("path\\ ", buffer), "\"path\\ \"") == 0);
+	assert(strcmp(esc_shell("C:\\My Documents\\", buffer), "\"C:\\My Documents\\\\\"") == 0);
+
+	/* backslash before embedded quote - backslash before quote must be doubled */
+	assert(strcmp(esc_shell("C:\\\"test\"", buffer), "\"C:\\\\\\\"test\\\"\"") == 0);
+	assert(strcmp(esc_shell("path\\\"file\"", buffer), "\"path\\\\\\\"file\\\"\"") == 0);
+
+	/* multiple trailing backslashes before end with quotes */
+	assert(strcmp(esc_shell("test\\\\ ", buffer), "\"test\\\\ \"") == 0);
+	assert(strcmp(esc_shell("path\\\\\\\\ ", buffer), "\"path\\\\\\\\ \"") == 0);
+
+	/* backslash NOT before quote - normal backslash */
+	assert(strcmp(esc_shell("test\\file ", buffer), "\"test\\file \"") == 0);
+	assert(strcmp(esc_shell("a\\b c", buffer), "\"a\\b c\"") == 0);
+
+	/* control characters - require quoting */
+	assert(strcmp(esc_shell("\x01", buffer), "\"\x01\"") == 0);
+	assert(strcmp(esc_shell("\x1F", buffer), "\"\x1F\"") == 0);
+	assert(strcmp(esc_shell("\x7F", buffer), "\"\x7F\"") == 0); /* DEL character */
+	assert(strcmp(esc_shell("test\x01test", buffer), "\"test\x01test\"") == 0);
+
+	/* complex real-world examples */
+	assert(strcmp(esc_shell("C:\\Program Files\\App", buffer), "\"C:\\Program Files\\App\"") == 0);
+	assert(strcmp(esc_shell("C:\\Program Files (x86)\\", buffer), "\"C:\\Program Files (x86)\\\\\"") == 0);
+	assert(strcmp(esc_shell("file (copy).txt", buffer), "\"file (copy).txt\"") == 0);
+	assert(strcmp(esc_shell("setup-v1.0.exe", buffer), "setup-v1.0.exe") == 0);
+	assert(strcmp(esc_shell("setup v1.0.exe", buffer), "\"setup v1.0.exe\"") == 0);
+
+	/* mixed quotes and special chars */
+	assert(strcmp(esc_shell("say \"hi\" & exit", buffer), "\"say \\\"hi\\\" & exit\"") == 0);
+	assert(strcmp(esc_shell("test \"a|b\"", buffer), "\"test \\\"a|b\\\"\"") == 0);
+
+	/* empty string */
+	assert(strcmp(esc_shell("", buffer), "") == 0);
+
+	/* all safe characters that don't need escaping */
+	assert(strcmp(esc_shell("abcdefghijklmnopqrstuvwxyz", buffer), "abcdefghijklmnopqrstuvwxyz") == 0);
+	assert(strcmp(esc_shell("ABCDEFGHIJKLMNOPQRSTUVWXYZ", buffer), "ABCDEFGHIJKLMNOPQRSTUVWXYZ") == 0);
+	assert(strcmp(esc_shell("0123456789", buffer), "0123456789") == 0);
+	assert(strcmp(esc_shell("._-+,@:", buffer), "._-+,@:") == 0);
 #else
-	assert(strcmp(esc_shell(",._+:@%%/-", buffer), ",._+:@%%/-") == 0);
+	/* basic cases - no special characters */
+	assert(strcmp(esc_shell("simple", buffer), "simple") == 0);
+	assert(strcmp(esc_shell("file.txt", buffer), "file.txt") == 0);
+	assert(strcmp(esc_shell("file123", buffer), "file123") == 0);
+	assert(strcmp(esc_shell("file_name-test.doc", buffer), "file_name-test.doc") == 0);
+	assert(strcmp(esc_shell(",._+:@/-", buffer), ",._+:@/-") == 0);
+	assert(strcmp(esc_shell("/usr/local/bin", buffer), "/usr/local/bin") == 0);
+
+	/* empty string */
+	assert(strcmp(esc_shell("", buffer), "") == 0);
+
+	/* space - escape with backslash */
 	assert(strcmp(esc_shell(" ", buffer), "\\ ") == 0);
+	assert(strcmp(esc_shell("file name.txt", buffer), "file\\ name.txt") == 0);
+	assert(strcmp(esc_shell("my document.doc", buffer), "my\\ document.doc") == 0);
+	assert(strcmp(esc_shell("  spaces  ", buffer), "\\ \\ spaces\\ \\ ") == 0);
+	assert(strcmp(esc_shell("a b c", buffer), "a\\ b\\ c") == 0);
+
+	/* tab - escape with backslash */
+	assert(strcmp(esc_shell("\t", buffer), "\\\t") == 0);
+	assert(strcmp(esc_shell("file\tname", buffer), "file\\\tname") == 0);
+	assert(strcmp(esc_shell("\t\t", buffer), "\\\t\\\t") == 0);
+
+	/* newline - escape with backslash */
+	assert(strcmp(esc_shell("\n", buffer), "\\\n") == 0);
+	assert(strcmp(esc_shell("line1\nline2", buffer), "line1\\\nline2") == 0);
+	assert(strcmp(esc_shell("\n\n", buffer), "\\\n\\\n") == 0);
+
+	/* carriage return - escape with backslash */
+	assert(strcmp(esc_shell("\r", buffer), "\\\r") == 0);
+	assert(strcmp(esc_shell("text\r\n", buffer), "text\\\r\\\n") == 0);
+
+	/* tilde (home directory expansion) */
+	assert(strcmp(esc_shell("~", buffer), "\\~") == 0);
+	assert(strcmp(esc_shell("~/file", buffer), "\\~/file") == 0);
+	assert(strcmp(esc_shell("file~name", buffer), "file\\~name") == 0);
+	assert(strcmp(esc_shell("~user", buffer), "\\~user") == 0);
+
+	/* backtick (command substitution) */
+	assert(strcmp(esc_shell("`", buffer), "\\`") == 0);
+	assert(strcmp(esc_shell("`command`", buffer), "\\`command\\`") == 0);
+	assert(strcmp(esc_shell("test`test", buffer), "test\\`test") == 0);
+	assert(strcmp(esc_shell("``", buffer), "\\`\\`") == 0);
+
+	/* hash (comment) */
+	assert(strcmp(esc_shell("#", buffer), "\\#") == 0);
+	assert(strcmp(esc_shell("#comment", buffer), "\\#comment") == 0);
+	assert(strcmp(esc_shell("file#name", buffer), "file\\#name") == 0);
+	assert(strcmp(esc_shell("test#123", buffer), "test\\#123") == 0);
+
+	/* dollar sign (variable expansion) */
+	assert(strcmp(esc_shell("$", buffer), "\\$") == 0);
+	assert(strcmp(esc_shell("$$", buffer), "\\$\\$") == 0);
+	assert(strcmp(esc_shell("$VAR", buffer), "\\$VAR") == 0);
+	assert(strcmp(esc_shell("${VAR}", buffer), "\\$\\{VAR\\}") == 0);
+	assert(strcmp(esc_shell("test$test", buffer), "test\\$test") == 0);
+	assert(strcmp(esc_shell("$1", buffer), "\\$1") == 0);
+	assert(strcmp(esc_shell("$PATH", buffer), "\\$PATH") == 0);
+
+	/* ampersand (background job) */
+	assert(strcmp(esc_shell("&", buffer), "\\&") == 0);
+	assert(strcmp(esc_shell("&&", buffer), "\\&\\&") == 0);
+	assert(strcmp(esc_shell("file&name", buffer), "file\\&name") == 0);
+	assert(strcmp(esc_shell("a&b&c", buffer), "a\\&b\\&c") == 0);
+	assert(strcmp(esc_shell("cmd1 & cmd2", buffer), "cmd1\\ \\&\\ cmd2") == 0);
+
+	/* asterisk (wildcard) */
+	assert(strcmp(esc_shell("*", buffer), "\\*") == 0);
+	assert(strcmp(esc_shell("**", buffer), "\\*\\*") == 0);
+	assert(strcmp(esc_shell("*.txt", buffer), "\\*.txt") == 0);
+	assert(strcmp(esc_shell("file*name", buffer), "file\\*name") == 0);
+	assert(strcmp(esc_shell("test*", buffer), "test\\*") == 0);
+
+	/* parentheses (subshell) */
+	assert(strcmp(esc_shell("(", buffer), "\\(") == 0);
+	assert(strcmp(esc_shell(")", buffer), "\\)") == 0);
+	assert(strcmp(esc_shell("()", buffer), "\\(\\)") == 0);
+	assert(strcmp(esc_shell("(test)", buffer), "\\(test\\)") == 0);
+	assert(strcmp(esc_shell("file(1)", buffer), "file\\(1\\)") == 0);
+	assert(strcmp(esc_shell("(a)(b)", buffer), "\\(a\\)\\(b\\)") == 0);
+
+	/* backslash (escape character) */
+	assert(strcmp(esc_shell("\\", buffer), "\\\\") == 0);
+	assert(strcmp(esc_shell("\\\\", buffer), "\\\\\\\\") == 0);
+	assert(strcmp(esc_shell("path\\to\\file", buffer), "path\\\\to\\\\file") == 0);
+	assert(strcmp(esc_shell("test\\test", buffer), "test\\\\test") == 0);
+	assert(strcmp(esc_shell("a\\b\\c", buffer), "a\\\\b\\\\c") == 0);
+
+	/* pipe (pipeline) */
+	assert(strcmp(esc_shell("|", buffer), "\\|") == 0);
+	assert(strcmp(esc_shell("||", buffer), "\\|\\|") == 0);
+	assert(strcmp(esc_shell("file|name", buffer), "file\\|name") == 0);
+	assert(strcmp(esc_shell("a|b|c", buffer), "a\\|b\\|c") == 0);
+	assert(strcmp(esc_shell("cmd1 | cmd2", buffer), "cmd1\\ \\|\\ cmd2") == 0);
+
+	/* square brackets (wildcard) */
+	assert(strcmp(esc_shell("[", buffer), "\\[") == 0);
+	assert(strcmp(esc_shell("]", buffer), "\\]") == 0);
+	assert(strcmp(esc_shell("[]", buffer), "\\[\\]") == 0);
+	assert(strcmp(esc_shell("[abc]", buffer), "\\[abc\\]") == 0);
+	assert(strcmp(esc_shell("file[1]", buffer), "file\\[1\\]") == 0);
+	assert(strcmp(esc_shell("[0-9]", buffer), "\\[0-9\\]") == 0);
+
+	/* curly braces (brace expansion) */
+	assert(strcmp(esc_shell("{", buffer), "\\{") == 0);
+	assert(strcmp(esc_shell("}", buffer), "\\}") == 0);
+	assert(strcmp(esc_shell("{}", buffer), "\\{\\}") == 0);
+	assert(strcmp(esc_shell("{a,b,c}", buffer), "\\{a,b,c\\}") == 0);
+	assert(strcmp(esc_shell("file{1,2}", buffer), "file\\{1,2\\}") == 0);
+	assert(strcmp(esc_shell("{1..10}", buffer), "\\{1..10\\}") == 0);
+
+	/* semicolon (command separator) */
+	assert(strcmp(esc_shell(";", buffer), "\\;") == 0);
+	assert(strcmp(esc_shell(";;", buffer), "\\;\\;") == 0);
+	assert(strcmp(esc_shell("cmd1;cmd2", buffer), "cmd1\\;cmd2") == 0);
+	assert(strcmp(esc_shell("test;test", buffer), "test\\;test") == 0);
+	assert(strcmp(esc_shell("a; b", buffer), "a\\;\\ b") == 0);
+
+	/* single quote */
+	assert(strcmp(esc_shell("'", buffer), "\\'") == 0);
+	assert(strcmp(esc_shell("''", buffer), "\\'\\'") == 0);
+	assert(strcmp(esc_shell("'test'", buffer), "\\'test\\'") == 0);
+	assert(strcmp(esc_shell("file'name", buffer), "file\\'name") == 0);
+	assert(strcmp(esc_shell("it's", buffer), "it\\'s") == 0);
+
+	/* double quote */
+	assert(strcmp(esc_shell("\"", buffer), "\\\"") == 0);
+	assert(strcmp(esc_shell("\"\"", buffer), "\\\"\\\"") == 0);
+	assert(strcmp(esc_shell("\"test\"", buffer), "\\\"test\\\"") == 0);
+	assert(strcmp(esc_shell("file\"name", buffer), "file\\\"name") == 0);
+	assert(strcmp(esc_shell("say \"hi\"", buffer), "say\\ \\\"hi\\\"") == 0);
+
+	/* angle brackets (redirection) */
+	assert(strcmp(esc_shell("<", buffer), "\\<") == 0);
+	assert(strcmp(esc_shell(">", buffer), "\\>") == 0);
+	assert(strcmp(esc_shell("<<", buffer), "\\<\\<") == 0);
+	assert(strcmp(esc_shell(">>", buffer), "\\>\\>") == 0);
+	assert(strcmp(esc_shell("a<b>c", buffer), "a\\<b\\>c") == 0);
+	assert(strcmp(esc_shell("file>output", buffer), "file\\>output") == 0);
+	assert(strcmp(esc_shell("cmd < in > out", buffer), "cmd\\ \\<\\ in\\ \\>\\ out") == 0);
+
+	/* question mark (wildcard) */
+	assert(strcmp(esc_shell("?", buffer), "\\?") == 0);
+	assert(strcmp(esc_shell("??", buffer), "\\?\\?") == 0);
+	assert(strcmp(esc_shell("file?.txt", buffer), "file\\?.txt") == 0);
+	assert(strcmp(esc_shell("test?test", buffer), "test\\?test") == 0);
+	assert(strcmp(esc_shell("file??", buffer), "file\\?\\?") == 0);
+
+	/* equals sign (assignment in some contexts) */
+	assert(strcmp(esc_shell("=", buffer), "\\=") == 0);
+	assert(strcmp(esc_shell("==", buffer), "\\=\\=") == 0);
+	assert(strcmp(esc_shell("VAR=value", buffer), "VAR\\=value") == 0);
+	assert(strcmp(esc_shell("a=b", buffer), "a\\=b") == 0);
+	assert(strcmp(esc_shell("PATH=/usr/bin", buffer), "PATH\\=/usr/bin") == 0);
+
+	/* exclamation mark (history expansion) */
+	assert(strcmp(esc_shell("!", buffer), "\\!") == 0);
+	assert(strcmp(esc_shell("!!", buffer), "\\!\\!") == 0);
+	assert(strcmp(esc_shell("test!test", buffer), "test\\!test") == 0);
+	assert(strcmp(esc_shell("!$", buffer), "\\!\\$") == 0);
+	assert(strcmp(esc_shell("!123", buffer), "\\!123") == 0);
+
+	/* control characters (0x01-0x1F) - escape with backslash */
+	assert(strcmp(esc_shell("\x01", buffer), "\\\x01") == 0);
+	assert(strcmp(esc_shell("\x02", buffer), "\\\x02") == 0);
+	assert(strcmp(esc_shell("\x1F", buffer), "\\\x1F") == 0);
+	assert(strcmp(esc_shell("test\x01test", buffer), "test\\\x01test") == 0);
+
+	/* DEL character (0x7F) */
+	assert(strcmp(esc_shell("\x7F", buffer), "\\\x7F") == 0);
+	assert(strcmp(esc_shell("test\x7Ftest", buffer), "test\\\x7Ftest") == 0);
+
+	/* multiple special characters combined */
+	assert(strcmp(esc_shell("$VAR & $OTHER", buffer), "\\$VAR\\ \\&\\ \\$OTHER") == 0);
+	assert(strcmp(esc_shell("*.txt | grep test", buffer), "\\*.txt\\ \\|\\ grep\\ test") == 0);
+	assert(strcmp(esc_shell("file (1) [copy].txt", buffer), "file\\ \\(1\\)\\ \\[copy\\].txt") == 0);
+	assert(strcmp(esc_shell("a & b | c", buffer), "a\\ \\&\\ b\\ \\|\\ c") == 0);
+	assert(strcmp(esc_shell("cmd1; cmd2 && cmd3", buffer), "cmd1\\;\\ cmd2\\ \\&\\&\\ cmd3") == 0);
+
+	/* complex real-world examples */
+	assert(strcmp(esc_shell("/home/user/My Documents", buffer), "/home/user/My\\ Documents") == 0);
+	assert(strcmp(esc_shell("/path/to/file (copy).txt", buffer), "/path/to/file\\ \\(copy\\).txt") == 0);
+	assert(strcmp(esc_shell("~/project/file-v1.0.tar.gz", buffer), "\\~/project/file-v1.0.tar.gz") == 0);
+	assert(strcmp(esc_shell("$(whoami)@$(hostname)", buffer), "\\$\\(whoami\\)@\\$\\(hostname\\)") == 0);
+	assert(strcmp(esc_shell("test && echo 'done'", buffer), "test\\ \\&\\&\\ echo\\ \\'done\\'") == 0);
+	assert(strcmp(esc_shell("file #1 [important].txt", buffer), "file\\ \\#1\\ \\[important\\].txt") == 0);
+	assert(strcmp(esc_shell("/tmp/test (1).txt", buffer), "/tmp/test\\ \\(1\\).txt") == 0);
+	assert(strcmp(esc_shell("var=$HOME/bin:$PATH", buffer), "var\\=\\$HOME/bin:\\$PATH") == 0);
+
+	/* edge cases with multiple escapes */
+	assert(strcmp(esc_shell("a\\ b", buffer), "a\\\\\\ b") == 0);
+	assert(strcmp(esc_shell("'\"test\"'", buffer), "\\'\\\"test\\\"\\'") == 0);
+	assert(strcmp(esc_shell("$(echo \"test\")", buffer), "\\$\\(echo\\ \\\"test\\\"\\)") == 0);
+
+	/* all safe characters that don't need escaping */
+	assert(strcmp(esc_shell("abcdefghijklmnopqrstuvwxyz", buffer), "abcdefghijklmnopqrstuvwxyz") == 0);
+	assert(strcmp(esc_shell("ABCDEFGHIJKLMNOPQRSTUVWXYZ", buffer), "ABCDEFGHIJKLMNOPQRSTUVWXYZ") == 0);
+	assert(strcmp(esc_shell("0123456789", buffer), "0123456789") == 0);
+	assert(strcmp(esc_shell("._-+,@:", buffer), "._-+,@:") == 0);
+	assert(strcmp(esc_shell("/path/to/file", buffer), "/path/to/file") == 0);
+	assert(strcmp(esc_shell("simple_file-name.txt", buffer), "simple_file-name.txt") == 0);
+
+	/* file extensions and versions */
+	assert(strcmp(esc_shell("file.tar.gz", buffer), "file.tar.gz") == 0);
+	assert(strcmp(esc_shell("app-v1.2.3.deb", buffer), "app-v1.2.3.deb") == 0);
+	assert(strcmp(esc_shell("test_2024-01-01.log", buffer), "test_2024-01-01.log") == 0);
 #endif
 
 	printf("Everything OK\n");
@@ -326,6 +692,7 @@ struct option long_options[] = {
 	{ "error-limit", 1, 0, 'L' },
 	{ "import", 1, 0, 'i' },
 	{ "log", 1, 0, 'l' },
+	{ "stats", 0, 0, 'A' },
 	{ "force-zero", 0, 0, 'Z' },
 	{ "force-empty", 0, 0, 'E' },
 	{ "force-uuid", 0, 0, 'U' },
@@ -333,12 +700,13 @@ struct option long_options[] = {
 	{ "force-nocopy", 0, 0, 'N' },
 	{ "force-full", 0, 0, 'F' },
 	{ "force-realloc", 0, 0, 'R' },
+	{ "bw-limit", 1, 0, 'w' },
 	{ "audit-only", 0, 0, 'a' },
 	{ "pre-hash", 0, 0, 'h' },
 	{ "speed-test", 0, 0, 'T' }, /* undocumented speed test command */
 	{ "gen-conf", 1, 0, 'C' },
 	{ "verbose", 0, 0, 'v' },
-	{ "quiet", 0, 0, 'q' }, /* undocumented quiet option */
+	{ "quiet", 0, 0, 'q' },
 	{ "gui", 0, 0, 'G' }, /* undocumented GUI interface option */
 	{ "help", 0, 0, 'H' },
 	{ "version", 0, 0, 'V' },
@@ -439,7 +807,7 @@ struct option long_options[] = {
 	{ "test-io-cache", 1, 0, OPT_TEST_IO_CACHE },
 
 	/* Print IO stats */
-	{ "test-io-stats", 0, 0, OPT_TEST_IO_STATS },
+	{ "test-io-stats", 0, 0, OPT_TEST_IO_STATS }, /* now replaced by -A, --stats */
 
 	/* Signal condition variable outside the mutex */
 	{ "test-cond-signal-outside", 0, 0, OPT_TEST_COND_SIGNAL_OUTSIDE },
@@ -484,7 +852,12 @@ struct option long_options[] = {
 };
 #endif
 
-#define OPTIONS "c:f:d:mebp:o:S:B:L:i:l:ZEUDNFRahTC:vqHVG"
+/*
+ * Free letters: gIjJkKMnPQrtuxXWz
+ *
+ * The 's' letter is used in main.c
+ */
+#define OPTIONS "c:f:d:mebp:o:S:B:L:i:l:AZEUDNFRahTC:vqHVGw:"
 
 volatile int global_interrupt = 0;
 
@@ -536,8 +909,9 @@ void signal_init(void)
 #define OPERATION_SPINDOWN 15
 #define OPERATION_DEVICES 16
 #define OPERATION_SMART 17
+#define OPERATION_PROBE 18
 
-int main(int argc, char* argv[])
+int snapraid_main(int argc, char* argv[])
 {
 	int c;
 	struct snapraid_option opt;
@@ -673,6 +1047,37 @@ int main(int argc, char* argv[])
 				/* LCOV_EXCL_STOP */
 			}
 			break;
+		case 'w' : /* --bw-limit */
+			if (optarg == 0) {
+				/* LCOV_EXCL_START */
+				log_fatal("Missing bandwidth limit\n");
+				exit(EXIT_FAILURE);
+				/* LCOV_EXCL_STOP */
+			}
+
+			/* Parse the number part */
+			opt.bwlimit = strtoul(optarg, &e, 10);
+			if (!e || e == optarg) {
+				/* LCOV_EXCL_START */
+				log_fatal("Invalid bandwidth limit '%s'\n", optarg);
+				exit(EXIT_FAILURE);
+				/* LCOV_EXCL_STOP */
+			}
+
+			/* Handle suffixes */
+			if ((e[0] == 'k' || e[0] == 'K') && e[1] == 0) {
+				opt.bwlimit *= 1000;
+			} else if ((e[0] == 'm' || e[0] == 'M') && e[1] == 0) {
+				opt.bwlimit *= 1000 * 1000;
+			} else if ((e[0] == 'g' || e[0] == 'G') && e[1] == 0) {
+				opt.bwlimit *= 1000 * 1000 * 1000;
+			} else if (e[0] != '\0') {
+				/* LCOV_EXCL_START */
+				log_fatal("Invalid bandwidth limit suffix '%s'\n", e);
+				exit(EXIT_FAILURE);
+				/* LCOV_EXCL_STOP */
+			}
+			break;
 		case 'S' :
 			blockstart = strtoul(optarg, &e, 0);
 			if (!e || *e) {
@@ -727,6 +1132,9 @@ int main(int argc, char* argv[])
 			}
 			log_file = optarg;
 			break;
+		case 'A' :
+			opt.force_stats = 1;
+			break;
 		case 'Z' :
 			opt.force_zero = 1;
 			break;
@@ -1005,6 +1413,8 @@ int main(int argc, char* argv[])
 		operation = OPERATION_DEVICES;
 	} else if (strcmp(argv[optind], "smart") == 0) {
 		operation = OPERATION_SMART;
+	} else if (strcmp(argv[optind], "probe") == 0) {
+		operation = OPERATION_PROBE;
 	} else {
 		/* LCOV_EXCL_START */
 		log_fatal("Unknown command '%s'\n", argv[optind]);
@@ -1029,6 +1439,7 @@ int main(int argc, char* argv[])
 	case OPERATION_FIX :
 	case OPERATION_CHECK :
 	case OPERATION_SMART :
+	case OPERATION_PROBE :
 	case OPERATION_DEVICES :
 	case OPERATION_SPINUP :
 	case OPERATION_SPINDOWN :
@@ -1174,7 +1585,6 @@ int main(int argc, char* argv[])
 	case OPERATION_REWRITE :
 	case OPERATION_READ :
 	case OPERATION_REHASH :
-	case OPERATION_SPINUP : /* we want to do it in different threads to avoid blocking */
 		/* avoid to check and access data disks if not needed */
 		opt.skip_disk_access = 1;
 		break;
@@ -1190,7 +1600,6 @@ int main(int argc, char* argv[])
 	case OPERATION_READ :
 	case OPERATION_REHASH :
 	case OPERATION_TOUCH :
-	case OPERATION_SPINUP : /* we want to do it in different threads to avoid blocking */
 		/* avoid to check and access parity disks if not needed */
 		opt.skip_parity_access = 1;
 		break;
@@ -1214,6 +1623,7 @@ int main(int argc, char* argv[])
 	case OPERATION_SPINDOWN :
 	case OPERATION_DEVICES :
 	case OPERATION_SMART :
+	case OPERATION_PROBE :
 		opt.skip_self = 1;
 		break;
 	}
@@ -1225,7 +1635,7 @@ int main(int argc, char* argv[])
 	case OPERATION_DRY :
 		break;
 #endif
-	default:
+	default :
 		/* we allow direct IO only on some commands */
 		if (opt.file_mode == ADVISE_DIRECT)
 			opt.file_mode = ADVISE_SEQUENTIAL;
@@ -1235,6 +1645,7 @@ int main(int argc, char* argv[])
 	switch (operation) {
 	case OPERATION_DEVICES :
 	case OPERATION_SMART :
+	case OPERATION_PROBE :
 		/* we may need to use these commands during operations */
 		opt.skip_lock = 1;
 		break;
@@ -1242,6 +1653,7 @@ int main(int argc, char* argv[])
 
 	switch (operation) {
 	case OPERATION_SMART :
+	case OPERATION_PROBE :
 		/* allow to run without configuration file */
 		opt.auto_conf = 1;
 		break;
@@ -1300,6 +1712,7 @@ int main(int argc, char* argv[])
 	(void)lock;
 #endif
 
+	ret = 0;
 	if (operation == OPERATION_DIFF) {
 		state_read(&state);
 
@@ -1372,21 +1785,6 @@ int main(int argc, char* argv[])
 			sleep(2);
 
 		ret = state_sync(&state, blockstart, blockcount);
-
-		/* save the new state if required */
-		if (!opt.kill_after_sync) {
-			if ((state.need_write || state.opt.force_content_write))
-				state_write(&state);
-		} else {
-			log_fatal("WARNING! Skipped writing state due to --test-kill-after-sync option.\n");
-		}
-
-		/* abort if required */
-		if (ret != 0) {
-			/* LCOV_EXCL_START */
-			exit(EXIT_FAILURE);
-			/* LCOV_EXCL_STOP */
-		}
 	} else if (operation == OPERATION_DRY) {
 		state_read(&state);
 
@@ -1399,7 +1797,7 @@ int main(int argc, char* argv[])
 		/* intercept signals while operating */
 		signal_init();
 
-		state_dry(&state, blockstart, blockcount);
+		ret = state_dry(&state, blockstart, blockcount);
 	} else if (operation == OPERATION_REHASH) {
 		state_read(&state);
 
@@ -1420,17 +1818,6 @@ int main(int argc, char* argv[])
 		signal_init();
 
 		ret = state_scrub(&state, plan, olderthan);
-
-		/* save the new state if required */
-		if (state.need_write || state.opt.force_content_write)
-			state_write(&state);
-
-		/* abort if required */
-		if (ret != 0) {
-			/* LCOV_EXCL_START */
-			exit(EXIT_FAILURE);
-			/* LCOV_EXCL_STOP */
-		}
 	} else if (operation == OPERATION_REWRITE) {
 		state_read(&state);
 
@@ -1447,14 +1834,14 @@ int main(int argc, char* argv[])
 	} else if (operation == OPERATION_TOUCH) {
 		state_read(&state);
 
+		memory();
+
 		state_touch(&state);
 
 		/* intercept signals while operating */
 		signal_init();
 
 		state_write(&state);
-
-		memory();
 	} else if (operation == OPERATION_SPINUP) {
 		state_device(&state, DEVICE_UP, &filterlist_disk);
 	} else if (operation == OPERATION_SPINDOWN) {
@@ -1463,6 +1850,8 @@ int main(int argc, char* argv[])
 		state_device(&state, DEVICE_LIST, 0);
 	} else if (operation == OPERATION_SMART) {
 		state_device(&state, DEVICE_SMART, 0);
+	} else if (operation == OPERATION_PROBE) {
+		state_device(&state, DEVICE_PROBE, 0);
 	} else if (operation == OPERATION_STATUS) {
 		state_read(&state);
 
@@ -1511,13 +1900,6 @@ int main(int argc, char* argv[])
 		} else { /* it's fix */
 			ret = state_check(&state, 1, blockstart, blockcount);
 		}
-
-		/* abort if required */
-		if (ret != 0) {
-			/* LCOV_EXCL_START */
-			exit(EXIT_FAILURE);
-			/* LCOV_EXCL_STOP */
-		}
 	}
 
 	/* close log file */
@@ -1541,6 +1923,22 @@ int main(int argc, char* argv[])
 	os_done();
 	lock_done();
 
+	/* abort if required */
+	if (ret != 0) {
+		/* LCOV_EXCL_START */
+		exit(EXIT_FAILURE);
+		/* LCOV_EXCL_STOP */
+	}
+	if (global_interrupt) {
+		/* LCOV_EXCL_START */
+#ifdef _WIN32
+		exit(STATUS_CONTROL_C_EXIT);
+#else
+		exit(128 + SIGINT);
+#endif
+		/* LCOV_EXCL_STOP */
+	}
+
 	return EXIT_SUCCESS;
 }
 
