@@ -80,6 +80,8 @@ void lock_done(void)
 
 int msg_level = 0;
 FILE* stdlog = 0;
+int msg_line_prev = 0; /**< Previous line width on stdout */
+int msg_line_curr = 0; /**< Current line width on stdout */
 
 /*
  * Note that in the following functions we always flush both
@@ -93,6 +95,57 @@ FILE* stdlog = 0;
  * caller should use log_flush().
  */
 
+static void vmsg(FILE* out, const char* format, va_list ap)
+{
+	char* dup = strdup_nofail(format);
+	int len = strlen(dup);
+	int written = 0;
+	char control = 0;
+
+	if (len > 0) {
+		if (dup[len - 1] == '\n') {
+			dup[len - 1] = 0;
+			control = '\n';
+		} else if (dup[len - 1] == '\r') {
+			dup[len - 1] = 0;
+			control = '\r';
+		}
+	}
+
+	if (dup[0]) {
+		written = vfprintf(out, dup, ap);
+	}
+
+	switch (control) {
+	case 0 :
+		msg_line_curr += written;
+		break;
+	case '\n' :
+		msg_line_curr += written;
+		/* writes spaces to overwrite any previous char */
+		while (msg_line_curr < msg_line_prev) {
+			fprintf(out, " ");
+			--msg_line_prev;
+		}
+		msg_line_prev = 0;
+		msg_line_curr = 0;
+		fprintf(out, "\n");
+		break;
+	case '\r' :
+		msg_line_curr += written;
+		/* writes spaces to overwrite any previous char */
+		while (msg_line_curr < msg_line_prev) {
+			fprintf(out, " ");
+			--msg_line_prev;
+		}
+		msg_line_prev = msg_line_curr;
+		msg_line_curr = 0;
+		fprintf(out, "\r");
+	}
+
+	free(dup);
+}
+
 void log_fatal(const char* format, ...)
 {
 	va_list ap;
@@ -100,18 +153,21 @@ void log_fatal(const char* format, ...)
 	lock_msg();
 
 	if (stdlog) {
-		va_start(ap, format);
 		fprintf(stdlog, "msg:fatal: ");
+
+		va_start(ap, format);
 		vfprintf(stdlog, format, ap);
-		fflush(stdlog);
 		va_end(ap);
+
+		fflush(stdlog);
 	}
 
 	va_start(ap, format);
-	vfprintf(stderr, format, ap);
-	fflush(stderr);
+	vmsg(stderr, format, ap);
 	va_end(ap);
 
+	fflush(stderr);
+
 	unlock_msg();
 }
 
@@ -122,16 +178,19 @@ void log_error(const char* format, ...)
 	lock_msg();
 
 	if (stdlog) {
-		va_start(ap, format);
 		fprintf(stdlog, "msg:error: ");
+
+		va_start(ap, format);
 		vfprintf(stdlog, format, ap);
-		fflush(stdlog);
 		va_end(ap);
+
+		fflush(stdlog);
 	} else {
 		va_start(ap, format);
-		vfprintf(stderr, format, ap);
-		fflush(stderr);
+		vmsg(stderr, format, ap);
 		va_end(ap);
+
+		fflush(stderr);
 	}
 
 	unlock_msg();
@@ -144,11 +203,13 @@ void log_expected(const char* format, ...)
 	lock_msg();
 
 	if (stdlog) {
-		va_start(ap, format);
 		fprintf(stdlog, "msg:expected: ");
+
+		va_start(ap, format);
 		vfprintf(stdlog, format, ap);
-		fflush(stdlog);
 		va_end(ap);
+
+		fflush(stdlog);
 	}
 
 	unlock_msg();
@@ -163,9 +224,9 @@ void log_tag(const char* format, ...)
 	if (stdlog) {
 		va_start(ap, format);
 		vfprintf(stdlog, format, ap);
-		/* here we intentionally don't flush */
-		/* to make the output faster */
 		va_end(ap);
+
+		/* here we intentionally don't flush to make the output faster */
 	}
 
 	unlock_msg();
@@ -190,17 +251,18 @@ void msg_status(const char* format, ...)
 	lock_msg();
 
 	if (stdlog) {
-		va_start(ap, format);
 		fprintf(stdlog, "msg:status: ");
+
+		va_start(ap, format);
 		vfprintf(stdlog, format, ap);
-		fflush(stdlog);
 		va_end(ap);
+
+		fflush(stdlog);
 	}
 
 	if (msg_level >= MSG_STATUS) {
 		va_start(ap, format);
-		vfprintf(stdout, format, ap);
-		fflush(stdout);
+		vmsg(stdout, format, ap);
 		va_end(ap);
 	}
 
@@ -218,9 +280,10 @@ void msg_info(const char* format, ...)
 
 	if (msg_level >= MSG_INFO) {
 		va_start(ap, format);
-		vfprintf(stdout, format, ap);
-		fflush(stdout);
+		vmsg(stdout, format, ap);
 		va_end(ap);
+
+		fflush(stdout);
 	}
 
 	unlock_msg();
@@ -233,18 +296,21 @@ void msg_progress(const char* format, ...)
 	lock_msg();
 
 	if (stdlog) {
-		va_start(ap, format);
 		fprintf(stdlog, "msg:progress: ");
+
+		va_start(ap, format);
 		vfprintf(stdlog, format, ap);
-		fflush(stdlog);
 		va_end(ap);
+
+		fflush(stdlog);
 	}
 
 	if (msg_level >= MSG_PROGRESS) {
 		va_start(ap, format);
-		vfprintf(stdout, format, ap);
-		fflush(stdout);
+		vmsg(stdout, format, ap);
 		va_end(ap);
+
+		fflush(stdout);
 	}
 
 	unlock_msg();
@@ -262,7 +328,7 @@ void msg_bar(const char* format, ...)
 
 	if (msg_level >= MSG_BAR) {
 		va_start(ap, format);
-		vfprintf(stdout, format, ap);
+		vmsg(stdout, format, ap);
 		va_end(ap);
 	}
 
@@ -276,18 +342,21 @@ void msg_verbose(const char* format, ...)
 	lock_msg();
 
 	if (stdlog) {
-		va_start(ap, format);
 		fprintf(stdlog, "msg:verbose: ");
+
+		va_start(ap, format);
 		vfprintf(stdlog, format, ap);
-		fflush(stdlog);
 		va_end(ap);
+
+		fflush(stdlog);
 	}
 
 	if (msg_level >= MSG_VERBOSE) {
 		va_start(ap, format);
-		vfprintf(stdout, format, ap);
-		fflush(stdout);
+		vmsg(stdout, format, ap);
 		va_end(ap);
+
+		fflush(stdout);
 	}
 
 	unlock_msg();
@@ -380,15 +449,12 @@ void printp(double v, size_t pad)
 	printl(buf, pad);
 }
 
-#define ESCAPE(from,escape,to) \
-	case from : \
+#define charcat(c) \
+	do { \
 		if (p == end) \
 			goto bail; \
-		*p++ = escape; \
-		if (p == end) \
-			goto bail; \
-		*p++ = to; \
-		break
+		*p++ = (c); \
+	} while (0)
 
 const char* esc_tag(const char* str, char* buffer)
 {
@@ -398,23 +464,29 @@ const char* esc_tag(const char* str, char* buffer)
 
 	/* copy string with escaping */
 	while (*str) {
-		char c = *str;
+		char c = *str++;
 
 		switch (c) {
-
-		ESCAPE('\n', '\\', 'n');
-		ESCAPE('\r', '\\', 'r');
-		ESCAPE(':', '\\', 'd');
-		ESCAPE('\\', '\\', '\\');
-
-		default:
-			if (p == end)
-				goto bail;
-			*p++ = c;
+		case '\n' :
+			charcat('\\');
+			charcat('n');
+			break;
+		case '\r' :
+			charcat('\\');
+			charcat('r');
+			break;
+		case ':' :
+			charcat('\\');
+			charcat('d');
+			break;
+		case '\\' :
+			charcat('\\');
+			charcat('\\');
+			break;
+		default :
+			charcat(c);
 			break;
 		}
-
-		++str;
 	}
 
 	/* put final 0 */
@@ -431,157 +503,113 @@ bail:
 	/* LCOV_EXCL_STOP */
 }
 
+#ifdef _WIN32
+static int needs_quote(const char* arg)
+{
+	while (*arg) {
+		char c = *arg;
+		if (c == ' ' || c == '\t' || c == '\n' || c == '\r' ||
+			c == '&' || c == '|' || c == '(' || c == ')' ||
+			c == '<' || c == '>' || c == '^' || c == '"' ||
+			c == '%' || c == '!' || c == '=' || c == ';' ||
+			(unsigned char)c < 32 || c == 127)
+			return 1;
+		++arg;
+	}
+
+	return 0;
+}
+#endif
+
+struct stream {
+	const char* str;
+	unsigned idx;
+	const char** map;
+	unsigned max;
+};
+
+static inline char ssget(struct stream* ss)
+{
+	/* get the next char */
+	char c = *ss->str++;
+
+	/* if one string is finished, go to the next */
+	while (c == 0 && ss->idx + 1 < ss->max) {
+		ss->str = ss->map[++ss->idx];
+		c = *ss->str++;
+	}
+
+	return c;
+}
+
+#ifdef _WIN32
 const char* esc_shell_multi(const char** str_map, unsigned str_max, char* buffer)
 {
 	char* begin = buffer;
 	char* end = begin + ESC_MAX;
 	char* p = begin;
-	unsigned str_mac;
-	const char* str;
+	struct stream ss;
+	char c;
+	unsigned i;
+	int has_quote;
 
-#ifdef _WIN32
-	int has_quote = 0;
-
-	for (str_mac = 0; str_mac < str_max; ++str_mac) {
-		str = str_map[str_mac];
-		if (strchr(str, ' ') != 0)
+	has_quote = 0;
+	for (i = 0; i < str_max; ++i) {
+		if (needs_quote(str_map[i]))
 			has_quote = 1;
 	}
 
-	if (has_quote) {
-		if (p == end)
-			goto bail;
-		*p++ = '"';
-	}
-#endif
+	ss.idx = 0;
+	ss.str = str_map[0];
+	ss.map = str_map;
+	ss.max = str_max;
 
-	/* copy string with escaping */
-	str_mac = 0;
-	str = str_map[str_mac];
-	while (1) {
-		/* get the next char */
-		char c = *str;
+	if (!has_quote) {
+		c = ssget(&ss);
+		while (c) {
+			charcat(c);
+			c = ssget(&ss);
+		}
+	} else {
+		/* starting quote */
+		charcat('"');
 
-		/* if one string is finished, go to the next */
-		while (c == 0 && str_mac + 1 < str_max) {
-			++str_mac;
-			str = str_map[str_mac];
-			c = *str;
+		c = ssget(&ss);
+		while (c) {
+			int bl = 0;
+			while (c == '\\') {
+				++bl;
+				c = ssget(&ss);
+			}
+
+			if (c == 0) {
+				/* double backslashes before closing quote */
+				bl = bl * 2;
+				while (bl--)
+					charcat('\\');
+				break;
+			} else if (c == '"') {
+				/* double backslashes + escape the quote */
+				bl = bl * 2 + 1;
+				while (bl--)
+					charcat('\\');
+				charcat('"');
+			} else {
+				/* normal backslashes */
+				while (bl--)
+					charcat('\\');
+				charcat(c);
+			}
+
+			c = ssget(&ss);
 		}
 
-		/* if we read all the strings, stop */
-		if (!c)
-			break;
-
-		switch (c) {
-#ifdef _WIN32
-		/*
-		 * Windows shell escape
-		 *
-		 * The Windows NT Command Shell
-		 * https://technet.microsoft.com/en-us/library/cc723564.aspx
-		 */
-		case '"' :
-			/* double quote, it needs to be quoted with \ */
-			if (has_quote) {
-				/* " -> "\"" -> (close quote)(quoted with \ ")(reopen quote) */
-				if (p == end)
-					goto bail;
-				*p++ = '"';
-				if (p == end)
-					goto bail;
-				*p++ = '\\';
-				if (p == end)
-					goto bail;
-				*p++ = '"';
-				if (p == end)
-					goto bail;
-				*p++ = '"';
-			} else {
-				/* " -> \" */
-				if (p == end)
-					goto bail;
-				*p++ = '\\';
-				if (p == end)
-					goto bail;
-				*p++ = '"';
-			}
-			break;
-		case '&' :
-		case '|' :
-		case '(' :
-		case ')' :
-		case '<' :
-		case '>' :
-		case '^' :
-			/* reserved chars, they need to be quoted with ^ */
-			if (has_quote) {
-				if (p == end)
-					goto bail;
-				*p++ = c;
-			} else {
-				if (p == end)
-					goto bail;
-				*p++ = '^';
-				if (p == end)
-					goto bail;
-				*p++ = c;
-			}
-			break;
-#else
-		/* special chars that need to be quoted */
-		case ' ' : /* space */
-		case '~' : /* home */
-		case '`' : /* command */
-		case '#' : /* comment */
-		case '$' : /* variable */
-		case '&' : /* background job */
-		case '*' : /* wildcard */
-		case '(' : /* shell */
-		case ')' : /* shell */
-		case '\\': /* quote */
-		case '|' : /* pipe */
-		case '[' : /* wildcard */
-		case ']' : /* wildcard */
-		case '{' : /* code */
-		case '}' : /* code */
-		case ';' : /* separator */
-		case '\'': /* quote */
-		case '"' : /* quote */
-		case '<' : /* redirect */
-		case '>' : /* redirect */
-		case '?' : /* wildcard */
-			if (p == end)
-				goto bail;
-			*p++ = '\\';
-			if (p == end)
-				goto bail;
-			*p++ = c;
-			break;
-#endif
-		default :
-			/* unquoted */
-			if (p == end)
-				goto bail;
-			*p++ = c;
-			break;
-		}
-
-		++str;
+		/* ending quote */
+		charcat('"');
 	}
 
-#ifdef _WIN32
-	if (has_quote) {
-		if (p == end)
-			goto bail;
-		*p++ = '"';
-	}
-#endif
-
 	/* put final 0 */
-	if (p == end)
-		goto bail;
-	*p = 0;
+	charcat(0);
 
 	return begin;
 
@@ -591,6 +619,79 @@ bail:
 	exit(EXIT_FAILURE);
 	/* LCOV_EXCL_STOP */
 }
+#else
+const char* esc_shell_multi(const char** str_map, unsigned str_max, char* buffer)
+{
+	char* begin = buffer;
+	char* end = begin + ESC_MAX;
+	char* p = begin;
+	struct stream ss;
+	char c;
+
+	ss.idx = 0;
+	ss.str = str_map[0];
+	ss.map = str_map;
+	ss.max = str_max;
+	
+	c = ssget(&ss);
+	while (c) {
+		switch (c) {
+		/* special chars that need to be quoted */
+		case ' ' : /* space */
+		case '\t' : /* tab */
+		case '\n' : /* newline */
+		case '\r' : /* carriage return */
+		case '~' : /* home */
+		case '`' : /* command */
+		case '#' : /* comment */
+		case '$' : /* variable */
+		case '&' : /* background job */
+		case '*' : /* wildcard */
+		case '(' : /* shell */
+		case ')' : /* shell */
+		case '\\' : /* quote */
+		case '|' : /* pipe */
+		case '[' : /* wildcard */
+		case ']' : /* wildcard */
+		case '{' : /* code */
+		case '}' : /* code */
+		case ';' : /* separator */
+		case '\'' : /* quote */
+		case '"' : /* quote */
+		case '<' : /* redirect */
+		case '>' : /* redirect */
+		case '?' : /* wildcard */
+		case '=' : /* assignment */
+		case '!' : /* history expansion */
+			charcat('\\');
+			charcat(c);
+			break;
+		default :
+			/* check for control characters */
+			if ((unsigned char)c < 32 || c == 127) {
+				charcat('\\');
+				charcat(c);
+			} else {
+				charcat(c);
+			}
+			break;
+		}
+
+		c = ssget(&ss);
+	}
+
+	/* put final 0 */
+	charcat(0);
+
+	return begin;
+
+bail:
+	/* LCOV_EXCL_START */
+	log_fatal("Escape for shell is too long\n");
+	exit(EXIT_FAILURE);
+	/* LCOV_EXCL_STOP */
+}
+#endif
 
 char* strpolish(char* s)
 {
@@ -631,6 +732,61 @@ unsigned strsplit(char** split_map, unsigned split_max, char* str, const char* d
 	return mac;
 }
 
+char* strtrim(char* str)
+{
+	char* begin;
+	char* end;
+
+	begin = str;
+	while (begin[0] && isspace((unsigned char)begin[0]))
+		++begin;
+
+	end = begin + strlen(begin);
+	while (end > begin && isspace((unsigned char)end[-1]))
+		--end;
+
+	end[0] = 0;
+
+	if (begin != end)
+		memmove(str, begin, end - begin + 1);
+
+	return str;
+}
+
+char* strlwr(char* str)
+{
+	char* s = str;
+
+	while (*s) {
+		*s = tolower((unsigned char)*s);
+		++s;
+	}
+	
+	return str;
+}
+
+char* worddigitstr(const char* haystack, const char* needle)
+{
+	size_t len = strlen(needle);
+	const char* s;
+
+	if (len == 0)
+		return NULL;
+
+	for (s = haystack; (s = strstr(s, needle)) != NULL; ++s) {
+
+		/* left boundary */
+		if (s == haystack || isspace((unsigned char)s[-1]) || isdigit((unsigned char)s[-1])) {
+			/* right boundary */
+			if (s[len] == '\0' || isspace((unsigned char)s[len]) || isdigit((unsigned char)s[len])) {
+				return (char *)s;
+			}
+		}
+	}
+
+	return NULL;
+}
+
 /****************************************************************************/
 /* path */
 
@@ -1002,7 +1158,21 @@ int advise_open(struct advise_struct* advise, int f)
 	) {
 		int ret;
 
-		/* advise sequential access */
+		/* advise noreuse access, this avoids to pollute the page cache */
+		/* supported from Linux Kernel 6.3 with this commit: https://github.com/torvalds/linux/commit/17e810229cb3068b692fa078bd9b3a6527e0866a */
+		ret = posix_fadvise(f, 0, 0, POSIX_FADV_NOREUSE);
+		if (ret == ENOSYS) {
+			/* call is not supported */
+			ret = 0;
+		}
+		if (ret != 0) {
+			/* LCOV_EXCL_START */
+			errno = ret; /* posix_fadvise return the error code */
+			return -1;
+			/* LCOV_EXCL_STOP */
+		}
+
+		/* advise sequential access, this doubles the read-ahead window size */
 		ret = posix_fadvise(f, 0, 0, POSIX_FADV_SEQUENTIAL);
 		if (ret == ENOSYS) {
 			/* call is not supported, like in armhf, see posix_fadvise manpage */
@@ -1498,7 +1668,6 @@ int smartctl_attribute(FILE* f, const char* file, const char* name, uint64_t* sm
 		} else if (sscanf(s, "Current Drive Temperature: %" SCNu64, &smart[SMART_TEMPERATURE_CELSIUS]) == 1) {
 		} else if (sscanf(s, "Drive Trip Temperature: %" SCNu64, &smart[SMART_AIRFLOW_TEMPERATURE_CELSIUS]) == 1) {
 		} else if (sscanf(s, "Accumulated start-stop cycles: %" SCNu64, &smart[SMART_START_STOP_COUNT]) == 1) {
-		} else if (sscanf(s, "Accumulated load-unload cycles: %" SCNu64, &smart[SMART_LOAD_CYCLE_COUNT]) == 1) {
 		} else if (sscanf(s, "  number of hours powered up = %" SCNu64, &smart[SMART_POWER_ON_HOURS]) == 1) {
 		/* ATA */
 		} else if (sscanf(s, "Serial Number: %63s", serial) == 1) {
@@ -1707,7 +1876,7 @@ void thread_cond_broadcast_and_unlock(thread_cond_t* cond, thread_mutex_t* mutex
 	}
 }
 
-void thread_create(thread_id_t* thread, void* (* func)(void *), void *arg)
+void thread_create(thread_id_t* thread, void* (*func)(void*), void *arg)
 {
 	if (pthread_create(thread, 0, func, arg) != 0) {
 		/* LCOV_EXCL_START */
@@ -1727,5 +1896,13 @@ void thread_join(thread_id_t thread, void** retval)
 	}
 }
 
+void thread_yield(void)
+{
+#ifdef __MINGW32__
+	Sleep(0);
+#else
+	sched_yield();
+#endif
+}
 #endif
 
