@@ -212,7 +212,7 @@ void printp(double v, size_t pad);
 /****************************************************************************/
 /* string */
 
-#define ESC_MAX (PATH_MAX*2 + 1)
+#define ESC_MAX (PATH_MAX * 2 + 1)
 
 /**
  * Escape a string for the log.
@@ -250,6 +250,24 @@ char* strpolish(char* s);
  */
 unsigned strsplit(char** split_map, unsigned split_max, char* line, const char* delimiters);
 
+/**
+ * Trim spaces from the start and the end
+ */
+char* strtrim(char* s);
+
+/**
+ * Lower case
+ */
+char* strlwr(char* s);
+
+/*
+ * Find the first occurrence of 'needle' in 'haystack' only if it
+ * appears as a separate word (space, number or string boundaries).
+ *
+ * Returns a pointer to the beginning of the match, or NULL if not found.
+ */
+char* worddigitstr(const char* haystack, const char* needle);
+
 /****************************************************************************/
 /* path */
 
@@ -439,6 +457,7 @@ void thread_cond_signal_and_unlock(thread_cond_t* cond, thread_mutex_t* mutex);
 void thread_cond_broadcast_and_unlock(thread_cond_t* cond, thread_mutex_t* mutex);
 void thread_create(thread_id_t* thread, void* (* func)(void *), void *arg);
 void thread_join(thread_id_t thread, void** retval);
+void thread_yield(void);
 #endif
 
 #endif
