@@ -341,7 +341,7 @@ static int state_hash_process(struct snapraid_state* state, block_off_t blocksta
 	}
 
 end:
-	state_progress_end(state, countpos, countmax, countsize);
+	state_progress_end(state, countpos, countmax, countsize, "Nothing to hash.\n");
 
 	/* note that at this point no io_error is possible */
 	/* because at the first one we bail out */
@@ -1298,6 +1298,31 @@ static int state_sync_process(struct snapraid_state* state, struct snapraid_pari
 			/* LCOV_EXCL_STOP */
 		}
 
+		/* thermal control */
+		if (state_thermal_alarm(state)) {
+			/* until now is misc */
+			state_usage_misc(state);
+
+			state_progress_stop(state);
+
+			/* before spinning down flush all the caches */
+			ret = state_flush(state, &io, parity_handle, blockcur);
+			if (ret == -1) {
+				/* LCOV_EXCL_START */
+				log_fatal("Stopping at block %u\n", blockcur);
+				++error;
+				goto bail;
+				/* LCOV_EXCL_STOP */
+			}
+
+			state_thermal_cooldown(state);
+
+			state_progress_restart(state);
+
+			/* drop until now */
+			state_usage_waste(state);
+		}
+
 		/* autosave */
 		if ((state->autosave != 0
 			&& autosavedone >= autosavelimit /* if we have reached the limit */
@@ -1316,18 +1341,13 @@ static int state_sync_process(struct snapraid_state* state, struct snapraid_pari
 
 			/* before writing the new content file we ensure that */
 			/* the parity is really written flushing the disk cache */
-			for (l = 0; l < state->level; ++l) {
-				ret = parity_sync(&parity_handle[l]);
-				if (ret == -1) {
-					/* LCOV_EXCL_START */
-					log_tag("parity_error:%u:%s: Sync error\n", blockcur, lev_config_name(l));
-					log_fatal("DANGER! Unexpected sync error in %s disk.\n", lev_name(l));
-					log_fatal("Ensure that disk '%s' is sane.\n", lev_config_name(l));
-					log_fatal("Stopping at block %u\n", blockcur);
-					++error;
-					goto bail;
-					/* LCOV_EXCL_STOP */
-				}
+			ret = state_flush(state, &io, parity_handle, blockcur);
+			if (ret == -1) {
+				/* LCOV_EXCL_START */
+				log_fatal("Stopping at block %u\n", blockcur);
+				++error;
+				goto bail;
+				/* LCOV_EXCL_STOP */
 			}
 
 			/* now we can safely write the content file */
@@ -1341,26 +1361,29 @@ static int state_sync_process(struct snapraid_state* state, struct snapraid_pari
 	}
 
 end:
-	state_progress_end(state, countpos, countmax, countsize);
-
-	state_usage_print(state);
+	state_progress_end(state, countpos, countmax, countsize, "Nothing to sync.\n");
 
 	/* before returning we ensure that */
 	/* the parity is really written flushing the disk cache */
-	for (l = 0; l < state->level; ++l) {
-		ret = parity_sync(&parity_handle[l]);
-		if (ret == -1) {
-			/* LCOV_EXCL_START */
-			log_tag("parity_error:%u:%s: Sync error\n", blockcur, lev_config_name(l));
-			log_fatal("DANGER! Unexpected sync error in %s disk.\n", lev_name(l));
-			log_fatal("Ensure that disk '%s' is sane.\n", lev_config_name(l));
-			log_fatal("Stopping at block %u\n", blockcur);
-			++error;
-			goto bail;
-			/* LCOV_EXCL_STOP */
-		}
+	ret = state_flush(state, &io, parity_handle, blockcur);
+	if (ret == -1) {
+		/* LCOV_EXCL_START */
+		log_fatal("Stopping at block %u\n", blockcur);
+		++error;
+		goto bail;
+		/* LCOV_EXCL_STOP */
 	}
 
+	/* save the new state if required */
+	if (!state->opt.kill_after_sync) {
+		if ((state->need_write || state->opt.force_content_write))
+			state_write(state);
+	} else {
+		log_fatal("WARNING! Skipped writing state due to --test-kill-after-sync option.\n");
+	}
+
+	state_usage_print(state);
+
 	if (error || silent_error || io_error) {
 		msg_status("\n");
 		msg_status("%8u file errors\n", error);
@@ -1592,7 +1615,7 @@ int state_sync(struct snapraid_state* state, block_off_t blockstart, block_off_t
 				/* LCOV_EXCL_STOP */
 			}
 		} else {
-			msg_status("Nothing to do\n");
+			msg_status("Nothing to sync.\n");
 		}
 	}
 
