@@ -68,6 +68,52 @@ const char* stat_desc(struct stat* st)
 	return "unknown";
 }
 
+/**
+ * Read a file from sys
+ *
+ * Return -1 on error, otherwise the data read
+ */
+#if HAVE_LINUX_DEVICE
+static int sysread(const char* path, char* buf, size_t buf_size)
+{
+	int f;
+	int ret;
+	int len;
+
+	f = open(path, O_RDONLY);
+	if (f == -1) {
+		/* LCOV_EXCL_START */
+		log_fatal("Failed to open '%s'.\n", path);
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	len = read(f, buf, buf_size);
+	if (len < 0) {
+		/* LCOV_EXCL_START */
+		close(f);
+		log_fatal("Failed to read '%s'.\n", path);
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	ret = close(f);
+	if (ret != 0) {
+		/* LCOV_EXCL_START */
+		log_fatal("Failed to close '%s'.\n", path);
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	if (len == (int)buf_size)
+		buf[buf_size - 1] = 0;
+	else
+		buf[len] = 0;
+
+	return len;
+}
+#endif
+
 /**
  * Get the device file from the device number.
  *
@@ -248,47 +294,22 @@ static int devdereference(uint64_t device, uint64_t* new_device)
 #if HAVE_LINUX_DEVICE
 static int tagread(const char* path, const char* tag, char* value, size_t value_size)
 {
-	int f;
 	int ret;
-	int len;
 	char buf[512];
 	size_t tag_len;
 	char* i;
 	char* e;
 
-	f = open(path, O_RDONLY);
-	if (f == -1) {
-		/* LCOV_EXCL_START */
-		log_fatal("Failed to open '%s'.\n", path);
-		return 0;
-		/* LCOV_EXCL_STOP */
-	}
-
-	len = read(f, buf, sizeof(buf));
-	if (len < 0) {
-		/* LCOV_EXCL_START */
-		close(f);
-		log_fatal("Failed to read '%s'.\n", path);
+	ret = sysread(path, buf, sizeof(buf));
+	if (ret < 0)
 		return -1;
-		/* LCOV_EXCL_STOP */
-	}
-	if (len == sizeof(buf)) {
+	if (ret == sizeof(buf)) {
 		/* LCOV_EXCL_START */
-		close(f);
 		log_fatal("Too long read '%s'.\n", path);
 		return -1;
 		/* LCOV_EXCL_STOP */
 	}
 
-	ret = close(f);
-	if (ret != 0) {
-		/* LCOV_EXCL_START */
-		log_fatal("Failed to close '%s'.\n", path);
-		return -1;
-		/* LCOV_EXCL_STOP */
-	}
-
-	buf[len] = 0;
 	tag_len = strlen(tag);
 
 	for (i = buf; *i; ++i) {
@@ -512,8 +533,54 @@ static int devuuid_blkid(uint64_t device, char* uuid, size_t uuid_size)
 }
 #endif
 
-int devuuid(uint64_t device, char* uuid, size_t uuid_size)
+#ifdef __APPLE__
+static int devuuid_darwin(const char* path, char* uuid, size_t uuid_size)
 {
+	CFStringRef path_apple = CFStringCreateWithCString(kCFAllocatorDefault, path, kCFStringEncodingUTF8);
+	DASessionRef session = DASessionCreate(kCFAllocatorDefault);
+
+	CFURLRef path_appler = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, path_apple, kCFURLPOSIXPathStyle, false);
+	DADiskRef disk;
+	do {
+		disk = DADiskCreateFromVolumePath(kCFAllocatorDefault, session, path_appler);
+		if (disk) {
+			CFRelease(path_appler);
+			break;
+		} else {
+			CFURLRef parent_path_appler = CFURLCreateCopyDeletingLastPathComponent(kCFAllocatorDefault, path_appler);
+			CFRelease(path_appler);
+			path_appler = parent_path_appler;
+		}
+	} while (true); // This is guaranteed to succeed eventually because it'll hit `/`.
+
+	CFDictionaryRef description = DADiskCopyDescription(disk);
+	CFUUIDRef uuid_apple = CFDictionaryGetValue(description, kDADiskDescriptionVolumeUUIDKey);
+	CFStringRef uuid_string = CFUUIDCreateString(kCFAllocatorDefault, uuid_apple);
+	bool success = CFStringGetCString(uuid_string, uuid, uuid_size, kCFStringEncodingUTF8);
+	CFRelease(uuid_string);
+	CFRelease(description);
+	CFRelease(disk);
+	CFRelease(session);
+	CFRelease(path_apple);
+
+	if (success) {
+		return 0;
+	} else {
+		return 1;
+	}
+}
+#endif
+
+int devuuid(uint64_t device, const char* path, char* uuid, size_t uuid_size)
+{
+	(void)path;
+	(void)device;
+
+#ifdef __APPLE__
+	if (devuuid_darwin(path, uuid, uuid_size) == 0)
+		return 0;
+#endif
+
 #if HAVE_LINUX_DEVICE
 	/* if the major is the null device */
 	if (major(device) == 0) {
@@ -902,7 +969,7 @@ static dev_t devread(const char* path)
  * Read a device tree filling the specified list of disk_t entries.
  */
 #if HAVE_LINUX_DEVICE
-static int devtree(const char* name, const char* custom, dev_t device, devinfo_t* parent, tommy_list* list)
+static int devtree(const char* name, const char* smartctl, const int* smartignore, dev_t device, devinfo_t* parent, tommy_list* list)
 {
 	char path[PATH_MAX];
 	DIR* d;
@@ -930,7 +997,7 @@ static int devtree(const char* name, const char* custom, dev_t device, devinfo_t
 					/* LCOV_EXCL_STOP */
 				}
 
-				if (devtree(name, custom, subdev, parent, list) != 0) {
+				if (devtree(name, smartctl, smartignore, subdev, parent, list) != 0) {
 					/* LCOV_EXCL_START */
 					closedir(d);
 					return -1;
@@ -975,7 +1042,8 @@ static int devtree(const char* name, const char* custom, dev_t device, devinfo_t
 
 		devinfo->device = device;
 		pathcpy(devinfo->name, sizeof(devinfo->name), name);
-		pathcpy(devinfo->smartctl, sizeof(devinfo->smartctl), custom);
+		pathcpy(devinfo->smartctl, sizeof(devinfo->smartctl), smartctl);
+		memcpy(devinfo->smartignore, smartignore, sizeof(devinfo->smartignore));
 		pathcpy(devinfo->file, sizeof(devinfo->file), path);
 		devinfo->parent = parent;
 
@@ -1072,7 +1140,7 @@ static int devscan(tommy_list* list)
  * Get SMART attributes.
  */
 #if HAVE_LINUX_DEVICE
-static int devsmart(dev_t device, const char* name, const char* custom, uint64_t* smart, char* serial, char* vendor, char* model)
+static int devsmart(dev_t device, const char* name, const char* smartctl, uint64_t* smart, char* serial, char* vendor, char* model)
 {
 	char cmd[PATH_MAX + 64];
 	char file[PATH_MAX];
@@ -1086,10 +1154,10 @@ static int devsmart(dev_t device, const char* name, const char* custom, uint64_t
 		/* LCOV_EXCL_STOP */
 	}
 
-	/* if there is a custom command */
-	if (custom[0]) {
+	/* if there is a custom smartctl command */
+	if (smartctl[0]) {
 		char option[PATH_MAX];
-		snprintf(option, sizeof(option), custom, file);
+		snprintf(option, sizeof(option), smartctl, file);
 		snprintf(cmd, sizeof(cmd), "smartctl -a %s", option);
 	} else {
 		snprintf(cmd, sizeof(cmd), "smartctl -a %s", file);
@@ -1137,10 +1205,10 @@ static int devsmart(dev_t device, const char* name, const char* custom, uint64_t
 #endif
 
 /**
- * Spin down a specific device.
+ * Get POWER state.
  */
 #if HAVE_LINUX_DEVICE
-static int devdown(dev_t device, const char* name, const char* custom)
+static int devprobe(dev_t device, const char* name, const char* smartctl, int* power)
 {
 	char cmd[PATH_MAX + 64];
 	char file[PATH_MAX];
@@ -1154,10 +1222,85 @@ static int devdown(dev_t device, const char* name, const char* custom)
 		/* LCOV_EXCL_STOP */
 	}
 
-	/* if there is a custom command */
-	if (custom[0]) {
+	/* if there is a custom smartctl command */
+	if (smartctl[0]) {
 		char option[PATH_MAX];
-		snprintf(option, sizeof(option), custom, file);
+		snprintf(option, sizeof(option), smartctl, file);
+		snprintf(cmd, sizeof(cmd), "smartctl -n standby,3 -i %s", option);
+	} else {
+		snprintf(cmd, sizeof(cmd), "smartctl -n standby,3 -i %s", file);
+	}
+
+	log_tag("smartctl:%s:%s:run: %s\n", file, name, cmd);
+
+	f = popen(cmd, "r");
+	if (!f) {
+		/* LCOV_EXCL_START */
+		log_fatal("Failed to run '%s' (from popen).\n", cmd);
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	if (smartctl_flush(f, file, name) != 0) {
+		/* LCOV_EXCL_START */
+		pclose(f);
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	ret = pclose(f);
+
+	log_tag("smartctl:%s:%s:ret: %x\n", file, name, ret);
+
+	if (!WIFEXITED(ret)) {
+		/* LCOV_EXCL_START */
+		log_fatal("Failed to run '%s' (not exited).\n", cmd);
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
+	if (WEXITSTATUS(ret) == 127) {
+		/* LCOV_EXCL_START */
+		log_fatal("Failed to run '%s' (from sh).\n", cmd);
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
+	if (WEXITSTATUS(ret) == 0) {
+		*power = POWER_ACTIVE;
+	} else if (WEXITSTATUS(ret) == 3) {
+		*power = POWER_STANDBY;
+	} else {
+		/* LCOV_EXCL_START */
+		log_fatal("Failed to run '%s' with return code %xh.\n", cmd, WEXITSTATUS(ret));
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	return 0;
+}
+#endif
+
+/**
+ * Spin down a specific device.
+ */
+#if HAVE_LINUX_DEVICE
+static int devdown(dev_t device, const char* name, const char* smartctl)
+{
+	char cmd[PATH_MAX + 64];
+	char file[PATH_MAX];
+	FILE* f;
+	int ret;
+
+	if (devresolve(device, file, sizeof(file)) != 0) {
+		/* LCOV_EXCL_START */
+		log_fatal("Failed to resolve device '%u:%u'.\n", major(device), minor(device));
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	/* if there is a custom smartctl command */
+	if (smartctl[0]) {
+		char option[PATH_MAX];
+		snprintf(option, sizeof(option), smartctl, file);
 		snprintf(cmd, sizeof(cmd), "smartctl -s standby,now %s", option);
 	} else {
 		snprintf(cmd, sizeof(cmd), "smartctl -s standby,now %s", file);
@@ -1208,33 +1351,81 @@ static int devdown(dev_t device, const char* name, const char* custom)
 #endif
 
 /**
- * Spin up a device.
- *
- * There isn't a defined way to spin up a device,
- * so we just do a generic write.
+ * Spin down a specific device if it's up.
  */
-static int devup(const char* mountpoint)
+#if HAVE_LINUX_DEVICE
+static int devdownifup(dev_t device, const char* name, const char* smartctl, int* power)
 {
+	*power = POWER_UNKNOWN;
+
+	if (devprobe(device, name, smartctl, power) != 0)
+		return -1;
+
+	if (*power == POWER_ACTIVE)
+		return devdown(device, name, smartctl);
+
+	return 0;
+}
+#endif
+
+/**
+ * Spin up a specific device.
+ */
+#if HAVE_LINUX_DEVICE
+static int devup(dev_t device)
+{
+	char file[PATH_MAX];
 	int ret;
-	char path[PATH_MAX];
+	int f;
+	void* buf;
 
-	/* add a temporary name used for writing */
-	pathprint(path, sizeof(path), "%s.snapraid-spinup", mountpoint);
-
-	/* do a generic write, and immediately undo it */
-	ret = mkdir(path, 0);
-	if (ret != 0 && errno != EEXIST) {
+	if (devresolve(device, file, sizeof(file)) != 0) {
 		/* LCOV_EXCL_START */
-		log_fatal("Failed to create dir '%s'.\n", path);
+		log_fatal("Failed to resolve device '%u:%u'.\n", major(device), minor(device));
 		return -1;
 		/* LCOV_EXCL_STOP */
 	}
 
-	/* remove the just created dir */
-	rmdir(path);
+	/* O_DIRECT requires memory aligned to the block size */
+	if (posix_memalign(&buf, 512, 512) != 0) {
+		/* LCOV_EXCL_START */
+		log_fatal("Failed to allocate aligned memory for device '%u:%u'.\n", major(device), minor(device));
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
 
+	f = open(file, O_RDONLY | O_DIRECT);
+	if (f < 0) {
+		/* LCOV_EXCL_START */
+		free(buf);
+		log_fatal("Failed to open device '%u:%u'.\n", major(device), minor(device));
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	ret = read(f, buf, 512);
+	if (ret < 0) {
+		/* LCOV_EXCL_START */
+		close(f);
+		free(buf);
+		log_fatal("Failed to read device '%u:%u'.\n", major(device), minor(device));
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	ret = close(f);
+	if (ret < 0) {
+		/* LCOV_EXCL_START */
+		free(buf);
+		log_fatal("Failed to close device '%u:%u'.\n", major(device), minor(device));
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	free(buf);
 	return 0;
 }
+#endif
 
 /**
  * Thread for spinning up.
@@ -1245,32 +1436,25 @@ static int devup(const char* mountpoint)
  */
 static void* thread_spinup(void* arg)
 {
+#if HAVE_LINUX_DEVICE
 	devinfo_t* devinfo = arg;
-	struct stat st;
 	uint64_t start;
 
 	start = tick_ms();
 
-	/* first get the device number, this usually doesn't trigger a thread_spinup */
-	if (stat(devinfo->mount, &st) != 0) {
-		/* LCOV_EXCL_START */
-		log_fatal("Failed to stat device '%s'.\n", devinfo->mount);
-		return (void*)-1;
-		/* LCOV_EXCL_STOP */
-	}
-
-	/* set the device number for printing */
-	devinfo->device = st.st_dev;
-
-	if (devup(devinfo->mount) != 0) {
+	if (devup(devinfo->device) != 0) {
 		/* LCOV_EXCL_START */
 		return (void*)-1;
 		/* LCOV_EXCL_STOP */
 	}
 
-	msg_status("Spunup device '%u:%u' for disk '%s' in %" PRIu64 " ms.\n", major(devinfo->device), minor(devinfo->device), devinfo->name, tick_ms() - start);
+	msg_status("Spunup device '%s' for disk '%s' in %" PRIu64 " ms.\n", devinfo->file, devinfo->name, tick_ms() - start);
 
 	return 0;
+#else
+	(void)arg;
+	return (void*)-1;
+#endif
 }
 
 /**
@@ -1299,6 +1483,34 @@ static void* thread_spindown(void* arg)
 #endif
 }
 
+/**
+ * Thread for spinning down.
+ */
+static void* thread_spindownifup(void* arg)
+{
+#if HAVE_LINUX_DEVICE
+	devinfo_t* devinfo = arg;
+	uint64_t start;
+	int power;
+
+	start = tick_ms();
+
+	if (devdownifup(devinfo->device, devinfo->name, devinfo->smartctl, &power) != 0) {
+		/* LCOV_EXCL_START */
+		return (void*)-1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	if (power == POWER_ACTIVE)
+		msg_status("Spundown device '%s' for disk '%s' in %" PRIu64 " ms.\n", devinfo->file, devinfo->name, tick_ms() - start);
+
+	return 0;
+#else
+	(void)arg;
+	return (void*)-1;
+#endif
+}
+
 /**
  * Thread for getting smart info.
  */
@@ -1320,6 +1532,27 @@ static void* thread_smart(void* arg)
 #endif
 }
 
+/**
+ * Thread for getting power info.
+ */
+static void* thread_probe(void* arg)
+{
+#if HAVE_LINUX_DEVICE
+	devinfo_t* devinfo = arg;
+
+	if (devprobe(devinfo->device, devinfo->name, devinfo->smartctl, &devinfo->power) != 0) {
+		/* LCOV_EXCL_START */
+		return (void*)-1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	return 0;
+#else
+	(void)arg;
+	return (void*)-1;
+#endif
+}
+
 static int device_thread(tommy_list* list, void* (*func)(void* arg))
 {
 	int fail = 0;
@@ -1362,73 +1595,58 @@ static int device_thread(tommy_list* list, void* (*func)(void* arg))
 
 int devquery(tommy_list* high, tommy_list* low, int operation, int others)
 {
-	tommy_node* i;
 	void* (*func)(void* arg) = 0;
 
+	if (operation == DEVICE_DOWN) {
+		/* flush all the disk caches before spinning down */
+		sync();
+	}
+
 #if HAVE_LINUX_DEVICE
-	if (operation != DEVICE_UP) {
-		struct stat st;
-		/* sysfs interface is required */
-		if (stat("/sys/dev/block", &st) != 0) {
+	tommy_node* i;
+	struct stat st;
+
+	/* sysfs interface is required */
+	if (stat("/sys/dev/block", &st) != 0) {
+		/* LCOV_EXCL_START */
+		log_fatal("Missing interface /sys/dev/block.\n");
+		return -1;
+		/* LCOV_EXCL_STOP */
+	}
+
+	/* for each device */
+	for (i = tommy_list_head(high); i != 0; i = i->next) {
+		devinfo_t* devinfo = i->data;
+		uint64_t device = devinfo->device;
+
+		/* if the major is the null device, find the real one */
+		if (major(device) == 0) {
+			/* obtain the real device */
+			if (devdereference(device, &device) != 0) {
+				/* LCOV_EXCL_START */
+				log_fatal("Failed to dereference device '%u:%u'.\n", major(device), minor(device));
+				return -1;
+				/* LCOV_EXCL_STOP */
+			}
+		}
+
+		/* get the device file */
+		if (devresolve(device, devinfo->file, sizeof(devinfo->file)) != 0) {
 			/* LCOV_EXCL_START */
-			log_fatal("Missing interface /sys/dev/block.\n");
+			log_fatal("Failed to resolve device '%u:%u'.\n", major(device), minor(device));
 			return -1;
 			/* LCOV_EXCL_STOP */
 		}
 
-		/* for each device */
-		for (i = tommy_list_head(high); i != 0; i = i->next) {
-			devinfo_t* devinfo = i->data;
-			uint64_t device = devinfo->device;
-
-			/* if the major is the null device, find the real one */
-			if (major(device) == 0) {
-				/* obtain the real device */
-				if (devdereference(device, &device) != 0) {
-					/* LCOV_EXCL_START */
-					log_fatal("Failed to dereference device '%u:%u'.\n", major(device), minor(device));
-					return -1;
-					/* LCOV_EXCL_STOP */
-				}
-			}
-
-			/* get the device file */
-			if (devresolve(device, devinfo->file, sizeof(devinfo->file)) != 0) {
-				/* LCOV_EXCL_START */
-				log_fatal("Failed to resolve device '%u:%u'.\n", major(device), minor(device));
-				return -1;
-				/* LCOV_EXCL_STOP */
-			}
-
-			/* expand the tree of devices */
-			if (devtree(devinfo->name, devinfo->smartctl, device, devinfo, low) != 0) {
-				/* LCOV_EXCL_START */
-				log_fatal("Failed to expand device '%u:%u'.\n", major(device), minor(device));
-				return -1;
-				/* LCOV_EXCL_STOP */
-			}
-		}
-	}
-#endif
-
-	if (operation == DEVICE_UP) {
-		/* duplicate the high */
-		for (i = tommy_list_head(high); i != 0; i = i->next) {
-			devinfo_t* devinfo = i->data;
-			devinfo_t* entry;
-
-			entry = calloc_nofail(1, sizeof(devinfo_t));
-
-			entry->device = devinfo->device;
-			pathcpy(entry->name, sizeof(entry->name), devinfo->name);
-			pathcpy(entry->mount, sizeof(entry->mount), devinfo->mount);
-
-			/* insert in the high */
-			tommy_list_insert_tail(low, &entry->node, entry);
+		/* expand the tree of devices */
+		if (devtree(devinfo->name, devinfo->smartctl, devinfo->smartignore, device, devinfo, low) != 0) {
+			/* LCOV_EXCL_START */
+			log_fatal("Failed to expand device '%u:%u'.\n", major(device), minor(device));
+			return -1;
+			/* LCOV_EXCL_STOP */
 		}
 	}
 
-#if HAVE_LINUX_DEVICE
 	/* add other devices */
 	if (others) {
 		if (devscan(low) != 0) {
@@ -1439,6 +1657,7 @@ int devquery(tommy_list* high, tommy_list* low, int operation, int others)
 		}
 	}
 #else
+	(void)high;
 	(void)others;
 #endif
 
@@ -1446,6 +1665,8 @@ int devquery(tommy_list* high, tommy_list* low, int operation, int others)
 	case DEVICE_UP : func = thread_spinup; break;
 	case DEVICE_DOWN : func = thread_spindown; break;
 	case DEVICE_SMART : func = thread_smart; break;
+	case DEVICE_PROBE : func = thread_probe; break;
+	case DEVICE_DOWNIFUP : func = thread_spindownifup; break;
 	}
 
 	if (!func)
@@ -1561,5 +1782,142 @@ size_t direct_size(void)
 	return size;
 }
 
+#if HAVE_LINUX_DEVICE
+
+/* List of possible ambient temperature labels */
+const char* AMBIENT_LABEL[] = {
+	"systin",
+	"auxtin",
+	"mb",
+	"board",
+	"motherboard",
+	"system",
+	"chassis",
+	"case",
+	"room",
+	"ambient",
+	0
+};
+
+int ambient_temperature(void)
+{
+	DIR* dir;
+	struct dirent* entry;
+	int lowest_temp = 0;
+
+	dir = opendir("/sys/class/hwmon");
+	if (!dir)
+		return 0;
+
+	/* iterate through hwmon devices */
+	while ((entry = readdir(dir)) != NULL) {
+		char path[PATH_MAX];
+		DIR* hwmon_dir;
+		struct dirent* hwmon_entry;
+
+		if (strncmp(entry->d_name, "hwmon", 5) != 0)
+			continue;
+
+		pathprint(path, sizeof(path), "/sys/class/hwmon/%s", entry->d_name);
+
+		/* iterate through temp*_input files */
+		hwmon_dir = opendir(path);
+		if (!hwmon_dir)
+			continue;
+
+		while ((hwmon_entry = readdir(hwmon_dir)) != NULL) {
+			char value[128];
+			char name[128];
+			char label[128];
+			char* dash;
+			int ret;
+			char* e;
+			int temp;
+
+			if (strncmp(hwmon_entry->d_name, "temp", 4) != 0)
+				continue;
+
+			dash = strrchr(hwmon_entry->d_name, '_');
+			if (dash == 0)
+				continue;
+
+			if (strcmp(dash, "_input") != 0)
+				continue;
+
+			/* read the temperature */
+			pathprint(path, sizeof(path), "/sys/class/hwmon/%s/%s", entry->d_name, hwmon_entry->d_name);
+
+			ret = sysread(path, value, sizeof(value));
+			if (ret < 0)
+				continue;
+
+			temp = strtol(value, &e, 10) / 1000;
+			if (*e != 0 && !isspace(*e))
+				continue;
+
+			/* cut the file name at "_input" */
+			*dash = 0;
+
+			/* read the corresponding name */
+			pathprint(path, sizeof(path), "/sys/class/hwmon/%s/name", entry->d_name);
+			ret = sysread(path, name, sizeof(name));
+			if (ret < 0) {
+				/* fallback to using the hwmon entry */
+				pathcpy(name, sizeof(name), entry->d_name);
+			}
+
+			/* read the corresponding label file */
+			pathprint(path, sizeof(path), "/sys/class/hwmon/%s/%s_label", entry->d_name, hwmon_entry->d_name);
+			ret = sysread(path, label, sizeof(label));
+			if (ret < 0) {
+				/* fallback to using the temp* name (e.g., temp1, temp2) */
+				pathcpy(label, sizeof(label), hwmon_entry->d_name);
+			}
+
+			/* trim spaces */
+			strtrim(name);
+			strtrim(label);
+
+			log_tag("thermal:ambient:device:%s:%s:%s:%s:%d\n", entry->d_name, name, hwmon_entry->d_name, label, temp);
+
+			/* check if temperature is in reasonable range */
+			if (temp < 15 || temp > 40)
+				continue;
+
+			/* lower case */
+			strlwr(label);
+
+			/* check if label matches possible ambient labels */
+			for (int i = 0; AMBIENT_LABEL[i]; ++i) {
+				if (worddigitstr(label, AMBIENT_LABEL[i]) != 0) {
+					log_tag("thermal:ambient:candidate:%d\n", temp);
+					if (lowest_temp == 0 || lowest_temp > temp)
+						lowest_temp = temp;
+					break;
+				}
+			}
+
+			/* accept also generic "temp1" */
+			if (strcmp(label, "temp1") == 0) {
+				log_tag("thermal:ambient:candidate:%d\n", temp);
+				if (lowest_temp == 0 || lowest_temp > temp)
+					lowest_temp = temp;
+			}
+		}
+
+		closedir(hwmon_dir);
+	}
+
+	closedir(dir);
+
+	return lowest_temp;
+}
+#else
+int ambient_temperature(void)
+{
+	return 0;
+}
+#endif
+
 #endif
 
